"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getApplySemigroup = exports.apT = exports.apS = exports.apSecond = exports.apFirst = exports.ap = void 0;
/**
 * The `Apply` class provides the `ap` which is used to apply a function to an argument under a type constructor.
 *
 * `Apply` can be used to lift functions of two or more arguments to work on values wrapped with the type constructor
 * `f`.
 *
 * Instances must satisfy the following law in addition to the `Functor` laws:
 *
 * 1. Associative composition: `fbc |> map(bc => ab => a => bc(ab(a))) |> ap(fab) <-> fbc |> ap(fab |> ap(fa))`
 *
 * Formally, `Apply` represents a strong lax semi-monoidal endofunctor.
 *
 * @since 3.0.0
 */
var function_1 = require("./function");
var Semigroup_1 = require("./Semigroup");
function ap(F, G) {
    return function (fa) {
        return function_1.flow(F.map(function (gab) { return function (ga) { return G.ap(ga)(gab); }; }), F.ap(fa));
    };
}
exports.ap = ap;
function apFirst(A) {
    return function (second) {
        return function_1.flow(A.map(function (a) { return function () { return a; }; }), A.ap(second));
    };
}
exports.apFirst = apFirst;
function apSecond(A) {
    return function (second) {
        return function_1.flow(A.map(function () { return function (b) { return b; }; }), A.ap(second));
    };
}
exports.apSecond = apSecond;
function apS(F) {
    return function (name, fb) {
        return function_1.flow(F.map(function (a) { return function (b) {
            var _a;
            return Object.assign({}, a, (_a = {}, _a[name] = b, _a));
        }; }), F.ap(fb));
    };
}
exports.apS = apS;
function apT(F) {
    return function (fb) { return function (fas) {
        return function_1.pipe(fas, F.map(function (a) { return function (b) { return __spreadArray(__spreadArray([], a), [b]); }; }), F.ap(fb));
    }; };
}
exports.apT = apT;
function getApplySemigroup(F) {
    return function (S) {
        var f = Semigroup_1.reverse(S).concat;
        return {
            concat: function (second) { return function (first) { return function_1.pipe(first, F.map(f), F.ap(second)); }; }
        };
    };
}
exports.getApplySemigroup = getApplySemigroup;
