"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getApplicative = exports.getApply = exports.flap = exports.Functor = exports.map = exports.Bifunctor = exports.Contravariant = exports.getBooleanAlgebra = exports.getHeytingAlgebra = exports.getRing = exports.getSemiring = exports.getMonoid = exports.getSemigroup = exports.getBounded = exports.getOrd = exports.getEq = exports.getShow = exports.mapLeft = exports.bimap = exports.contramap = exports.make = void 0;
var Bifunctor_1 = require("./Bifunctor");
var function_1 = require("./function");
var Functor_1 = require("./Functor");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
exports.make = function_1.unsafeCoerce;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Contravariant
 * @since 3.0.0
 */
var contramap = function () { return function_1.unsafeCoerce; };
exports.contramap = contramap;
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
var bimap = function (f) { return function_1.flow(f, exports.make); };
exports.bimap = bimap;
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
exports.mapLeft = 
/*#__PURE__*/
Bifunctor_1.mapLeftDefault(exports.bimap);
/**
 * @category instances
 * @since 3.0.0
 */
var getShow = function (S) { return ({
    show: function (c) { return "make(" + S.show(c) + ")"; }
}); };
exports.getShow = getShow;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getEq = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getOrd = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getBounded = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getSemigroup = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getMonoid = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getSemiring = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getRing = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getHeytingAlgebra = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.getBooleanAlgebra = function_1.identity;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Contravariant = {
    contramap: exports.contramap
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Bifunctor = {
    bimap: exports.bimap,
    mapLeft: exports.mapLeft
};
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
exports.map = 
/*#__PURE__*/
Bifunctor_1.map(exports.Bifunctor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = 
/*#__PURE__*/
Functor_1.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
var getApply = function (S) { return ({
    map: exports.map,
    ap: function (fa) { return function (fab) { return exports.make(S.concat(fa)(fab)); }; }
}); };
exports.getApply = getApply;
/**
 * @category instances
 * @since 3.0.0
 */
var getApplicative = function (M) {
    var A = exports.getApply(M);
    return {
        map: A.map,
        ap: A.ap,
        of: function () { return exports.make(M.empty); }
    };
};
exports.getApplicative = getApplicative;
