"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reduceRightWithIndex = exports.foldMapWithIndex = exports.reduceWithIndex = void 0;
/**
 * A `Foldable` with an additional index.
 *
 * A `FoldableWithIndex` instance must be compatible with its `Foldable` instance:
 *
 * ```ts
 * reduce(b, f) = reduceWithIndex(b, (_, b, a) => f(b, a))
 * foldMap(M)(f) = foldMapWithIndex(M)((_, a) => f(a))
 * reduceRight(b, f) = reduceRightWithIndex(b, (_, a, b) => f(a, b))
 * ```
 *
 * @since 3.0.0
 */
var function_1 = require("./function");
function reduceWithIndex(F, G) {
    return function (b, f) {
        return F.reduceWithIndex(b, function (i, b, ga) {
            return function_1.pipe(ga, G.reduceWithIndex(b, function (j, b, a) { return f([i, j], b, a); }));
        });
    };
}
exports.reduceWithIndex = reduceWithIndex;
function foldMapWithIndex(F, G) {
    return function (M) {
        var foldMapWithIndexF = F.foldMapWithIndex(M);
        var foldMapWithIndexG = G.foldMapWithIndex(M);
        return function (f) {
            return foldMapWithIndexF(function (i, ga) {
                return function_1.pipe(ga, foldMapWithIndexG(function (j, a) { return f([i, j], a); }));
            });
        };
    };
}
exports.foldMapWithIndex = foldMapWithIndex;
function reduceRightWithIndex(F, G) {
    return function (b, f) {
        return F.reduceRightWithIndex(b, function (i, ga, b) {
            return function_1.pipe(ga, G.reduceRightWithIndex(b, function (j, a, b) { return f([i, j], a, b); }));
        });
    };
}
exports.reduceRightWithIndex = reduceRightWithIndex;
