"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tupled = exports.bindTo = exports.map = exports.flap = void 0;
/**
 * A `Functor` is a type constructor which supports a mapping operation `map`.
 *
 * `map` can be used to turn functions `A -> B` into functions `F<A> -> F<B>` whose argument and return types use the type
 * constructor `F` to represent some computational context.
 *
 * Instances must satisfy the following laws:
 *
 * 1. Identity: `map(identity) <-> identity`
 * 2. Composition: `map(flow(ab, bc)) <-> flow(map(ab), map(bc))`
 *
 * @since 3.0.0
 */
var function_1 = require("./function");
var tuple_1 = require("./tuple");
function flap(F) {
    return function (a) { return F.map(function_1.apply(a)); };
}
exports.flap = flap;
function map(F, G) {
    return function_1.flow(G.map, F.map);
}
exports.map = map;
function bindTo(F) {
    return function (name) { return F.map(function (a) {
        var _a;
        return (_a = {}, _a[name] = a, _a);
    }); };
}
exports.bindTo = bindTo;
function tupled(F) {
    return F.map(tuple_1.tuple);
}
exports.tupled = tupled;
