"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapWithIndex = void 0;
/**
 * A `FunctorWithIndex` is a type constructor which supports a mapping operation `mapWithIndex`.
 *
 * `mapWithIndex` can be used to turn functions `i -> a -> b` into functions `f a -> f b` whose argument and return types use the type
 * constructor `f` to represent some computational context.
 *
 * Instances must satisfy the following laws:
 *
 * 1. Identity: `F.mapWithIndex((_i, a) => a) <-> fa`
 * 2. Composition: `F.mapWithIndex((_i, a) => bc(ab(a))) <-> flow(F.mapWithIndex((_i, a) => ab(a)), F.mapWithIndex((_i, b) => bc(b)))`
 *
 * @since 3.0.0
 */
var function_1 = require("./function");
function mapWithIndex(F, G) {
    return function (f) {
        return F.mapWithIndex(function (i, ga) {
            return function_1.pipe(ga, G.mapWithIndex(function (j, a) { return f([i, j], a); }));
        });
    };
}
exports.mapWithIndex = mapWithIndex;
