"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toReadonlyArray = exports.reduceRight = exports.foldMap = exports.reduce = exports.elem = exports.isSubset = exports.every = exports.some = exports.size = exports.isEmpty = exports.empty = exports.getDifferenceMagma = exports.getIntersectionSemigroup = exports.getUnionMonoid = exports.getUnionSemigroup = exports.getEq = exports.getShow = exports.partitionMap = exports.partition = exports.filterMap = exports.filter = exports.separate = exports.compact = exports.chain = exports.map = exports.remove = exports.insert = exports.difference = exports.intersection = exports.union = exports.toggle = exports.fromReadonlyArray = exports.singleton = void 0;
var Eq_1 = require("./Eq");
var function_1 = require("./function");
var _ = __importStar(require("./internal"));
var Predicate_1 = require("./Predicate");
var Separated_1 = require("./Separated");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Create a `ReadonlySet` from one element.
 *
 * @category constructors
 * @since 3.0.0
 */
var singleton = function (a) { return new Set([a]); };
exports.singleton = singleton;
/**
 * Create a `ReadonlySet` from a `ReadonlyArray`.
 *
 * @category constructors
 * @since 3.0.0
 */
var fromReadonlyArray = function (E) { return function (as) {
    var len = as.length;
    var r = new Set();
    var has = exports.elem(E);
    for (var i = 0; i < len; i++) {
        var a = as[i];
        if (!has(a)(r)) {
            r.add(a);
        }
    }
    return r;
}; };
exports.fromReadonlyArray = fromReadonlyArray;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Checks an element is a member of a set;
 * If yes, removes the value from the set
 * If no, inserts the value to the set
 *
 * @category combinators
 * @since 3.0.0
 */
var toggle = function (E) {
    var elemE = exports.elem(E);
    var removeE = exports.remove(E);
    var insertE = exports.insert(E);
    return function (a) {
        var elemEa = elemE(a);
        return function (set) { return (elemEa(set) ? removeE : insertE)(a)(set); };
    };
};
exports.toggle = toggle;
/**
 * Return the union of two `ReadonlySet`s.
 *
 * @category combinators
 * @since 3.0.0
 */
var union = function (E) {
    var elemE = exports.elem(E);
    return function (second) { return function (first) {
        if (exports.isEmpty(first)) {
            return second;
        }
        if (exports.isEmpty(second)) {
            return first;
        }
        var r = new Set(first);
        second.forEach(function (e) {
            if (!elemE(e)(r)) {
                r.add(e);
            }
        });
        return r;
    }; };
};
exports.union = union;
/**
 * The `ReadonlySet` of elements which are in both the first and second `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
var intersection = function (E) {
    var elemE = exports.elem(E);
    return function (second) { return function (first) {
        if (exports.isEmpty(first) || exports.isEmpty(second)) {
            return exports.empty;
        }
        var r = new Set();
        first.forEach(function (e) {
            if (elemE(e)(second)) {
                r.add(e);
            }
        });
        return r;
    }; };
};
exports.intersection = intersection;
/**
 * Return the set difference (`x` - `y`).
 *
 * @example
 * import { difference } from 'fp-ts/ReadonlySet'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(new Set([1, 2]), difference(N.Eq)(new Set([1, 3]))), new Set([2]))
 *
 * @category combinators
 * @since 3.0.0
 */
var difference = function (E) {
    var elemE = exports.elem(E);
    return function (second) { return filter(function (a) { return !elemE(a)(second); }); };
};
exports.difference = difference;
/**
 * Insert a value into a `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
var insert = function (E) {
    var elemE = exports.elem(E);
    return function (a) { return function (set) {
        if (!elemE(a)(set)) {
            var r = new Set(set);
            r.add(a);
            return r;
        }
        else {
            return set;
        }
    }; };
};
exports.insert = insert;
/**
 * Delete a value from a `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
var remove = function (E) { return function (a) { return filter(Predicate_1.not(E.equals(a))); }; };
exports.remove = remove;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Projects a `ReadonlySet` through a function.
 *
 * @category Functor
 * @since 3.0.0
 */
var map = function (E) {
    var elemE = exports.elem(E);
    return function (f) { return function (set) {
        var r = new Set();
        set.forEach(function (e) {
            var v = f(e);
            if (!elemE(v)(r)) {
                r.add(v);
            }
        });
        return r;
    }; };
};
exports.map = map;
/**
 * @category Chain
 * @since 3.0.0
 */
var chain = function (E) {
    var elemE = exports.elem(E);
    return function (f) { return function (set) {
        var r = new Set();
        set.forEach(function (e) {
            f(e).forEach(function (e) {
                if (!elemE(e)(r)) {
                    r.add(e);
                }
            });
        });
        return r;
    }; };
};
exports.chain = chain;
/**
 * @category Compactable
 * @since 3.0.0
 */
var compact = function (E) { return exports.filterMap(E)(function_1.identity); };
exports.compact = compact;
/**
 * @category Compactable
 * @since 3.0.0
 */
var separate = function (EE, EA) { return function (fa) {
    var elemEE = exports.elem(EE);
    var elemEA = exports.elem(EA);
    var left = new Set();
    var right = new Set();
    fa.forEach(function (e) {
        switch (e._tag) {
            case 'Left':
                if (!elemEE(e.left)(left)) {
                    left.add(e.left);
                }
                break;
            case 'Right':
                if (!elemEA(e.right)(right)) {
                    right.add(e.right);
                }
                break;
        }
    });
    return Separated_1.separated(left, right);
}; };
exports.separate = separate;
function filter(predicate) {
    return function (s) {
        var values = s.values();
        var e;
        var r = new Set();
        while (!(e = values.next()).done) {
            var a = e.value;
            if (predicate(a)) {
                r.add(a);
            }
        }
        return r;
    };
}
exports.filter = filter;
/**
 * @category Filterable
 * @since 3.0.0
 */
var filterMap = function (E) {
    var elemE = exports.elem(E);
    return function (f) { return function (fa) {
        var r = new Set();
        fa.forEach(function (a) {
            var ob = f(a);
            if (_.isSome(ob) && !elemE(ob.value)(r)) {
                r.add(ob.value);
            }
        });
        return r;
    }; };
};
exports.filterMap = filterMap;
function partition(predicate) {
    return function (s) {
        var values = s.values();
        var e;
        var right = new Set();
        var left = new Set();
        while (!(e = values.next()).done) {
            var a = e.value;
            if (predicate(a)) {
                right.add(a);
            }
            else {
                left.add(a);
            }
        }
        return Separated_1.separated(left, right);
    };
}
exports.partition = partition;
/**
 * @category Filterable
 * @since 3.0.0
 */
var partitionMap = function (EB, EC) { return function (f) { return function (s) {
    var values = s.values();
    var e;
    var left = new Set();
    var right = new Set();
    var hasB = exports.elem(EB);
    var hasC = exports.elem(EC);
    while (!(e = values.next()).done) {
        var v = f(e.value);
        switch (v._tag) {
            case 'Left':
                if (!hasB(v.left)(left)) {
                    left.add(v.left);
                }
                break;
            case 'Right':
                if (!hasC(v.right)(right)) {
                    right.add(v.right);
                }
                break;
        }
    }
    return Separated_1.separated(left, right);
}; }; };
exports.partitionMap = partitionMap;
/**
 * @category instances
 * @since 3.0.0
 */
var getShow = function (S) { return ({
    show: function (s) {
        var entries = [];
        s.forEach(function (a) {
            entries.push(S.show(a));
        });
        return "new Set([" + entries.sort().join(', ') + "])";
    }
}); };
exports.getShow = getShow;
/**
 * @category instances
 * @since 3.0.0
 */
var getEq = function (E) {
    var subsetE = exports.isSubset(E);
    return Eq_1.fromEquals(function (second) { return function (first) { return subsetE(first)(second) && subsetE(second)(first); }; });
};
exports.getEq = getEq;
/**
 * @category instances
 * @since 3.0.0
 */
var getUnionSemigroup = function (E) { return ({
    concat: exports.union(E)
}); };
exports.getUnionSemigroup = getUnionSemigroup;
/**
 * @category instances
 * @since 3.0.0
 */
var getUnionMonoid = function (E) { return ({
    concat: exports.getUnionSemigroup(E).concat,
    empty: exports.empty
}); };
exports.getUnionMonoid = getUnionMonoid;
/**
 * @category instances
 * @since 3.0.0
 */
var getIntersectionSemigroup = function (E) { return ({
    concat: exports.intersection(E)
}); };
exports.getIntersectionSemigroup = getIntersectionSemigroup;
/**
 * @category instances
 * @since 3.0.0
 */
var getDifferenceMagma = function (E) { return ({
    concat: exports.difference(E)
}); };
exports.getDifferenceMagma = getDifferenceMagma;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * An empty `ReadonlySet`.
 *
 * @since 3.0.0
 */
exports.empty = new Set();
/**
 * Test whether a `ReadonlySet` is empty.
 *
 * @since 3.0.0
 */
var isEmpty = function (set) { return set.size === 0; };
exports.isEmpty = isEmpty;
/**
 * Calculate the number of elements in a `ReadonlySet`.
 *
 * @since 3.0.0
 */
var size = function (set) { return set.size; };
exports.size = size;
/**
 * @since 3.0.0
 */
var some = function (predicate) { return function (s) {
    var values = s.values();
    var e;
    var found = false;
    while (!found && !(e = values.next()).done) {
        found = predicate(e.value);
    }
    return found;
}; };
exports.some = some;
function every(predicate) {
    return Predicate_1.not(exports.some(Predicate_1.not(predicate)));
}
exports.every = every;
/**
 * Return `true` if and only if every element in the first `ReadonlySet` is an element of the second `ReadonlySet`.
 *
 * @since 3.0.0
 */
var isSubset = function (E) {
    var elemE = exports.elem(E);
    return function (second) { return every(function (a) { return elemE(a)(second); }); };
};
exports.isSubset = isSubset;
/**
 * Tests whether a value is a member of a `ReadonlySet`.
 *
 * @since 3.0.0
 */
var elem = function (E) { return function (a) {
    var predicate = E.equals(a);
    return function (set) {
        var values = set.values();
        var e;
        var found = false;
        while (!found && !(e = values.next()).done) {
            found = predicate(e.value);
        }
        return found;
    };
}; };
exports.elem = elem;
/**
 * @since 3.0.0
 */
var reduce = function (O) {
    var toReadonlyArrayO = exports.toReadonlyArray(O);
    return function (b, f) { return function (fa) { return toReadonlyArrayO(fa).reduce(f, b); }; };
};
exports.reduce = reduce;
/**
 * @since 3.0.0
 */
var foldMap = function (O) { return function (M) {
    var toReadonlyArrayO = exports.toReadonlyArray(O);
    return function (f) { return function (fa) { return toReadonlyArrayO(fa).reduce(function (b, a) { return M.concat(f(a))(b); }, M.empty); }; };
}; };
exports.foldMap = foldMap;
/**
 * @since 3.0.0
 */
var reduceRight = function (O) {
    var toReadonlyArrayO = exports.toReadonlyArray(O);
    return function (b, f) { return function (fa) { return toReadonlyArrayO(fa).reduceRight(function (b, a) { return f(a, b); }, b); }; };
};
exports.reduceRight = reduceRight;
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlySet`.
 *
 * @since 3.0.0
 */
var toReadonlyArray = function (O) { return function (s) {
    var out = [];
    s.forEach(function (e) { return out.push(e); });
    return out.sort(function (first, second) { return O.compare(second)(first); });
}; };
exports.toReadonlyArray = toReadonlyArray;
