"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.experiment = exports.peeks = exports.seeks = exports.seek = exports.Comonad = exports.flap = exports.Functor = exports.duplicate = exports.extract = exports.extend = exports.map = void 0;
var function_1 = require("./function");
var Functor_1 = require("./Functor");
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
var map = function (f) { return function (fa) { return ({
    peek: function (s) { return f(fa.peek(s)); },
    pos: fa.pos
}); }; };
exports.map = map;
/**
 * @category Extend
 * @since 3.0.0
 */
var extend = function (f) { return function (wa) { return ({
    peek: function (s) { return f({ peek: wa.peek, pos: s }); },
    pos: wa.pos
}); }; };
exports.extend = extend;
/**
 * @category Extract
 * @since 3.0.0
 */
var extract = function (wa) { return wa.peek(wa.pos); };
exports.extract = extract;
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
exports.duplicate = 
/*#__PURE__*/
exports.extend(function_1.identity);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = 
/*#__PURE__*/
Functor_1.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Comonad = {
    map: exports.map,
    extend: exports.extend,
    extract: exports.extract
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Reposition the focus at the specified position.
 *
 * @since 3.0.0
 */
var seek = function (s) { return function (wa) { return ({ peek: wa.peek, pos: s }); }; };
exports.seek = seek;
/**
 * Reposition the focus at the specified position, which depends on the current position.
 *
 * @since 3.0.0
 */
var seeks = function (f) { return function (wa) { return ({ peek: wa.peek, pos: f(wa.pos) }); }; };
exports.seeks = seeks;
/**
 * Extract a value from a position which depends on the current position.
 *
 * @since 3.0.0
 */
var peeks = function (f) { return function (wa) { return wa.peek(f(wa.pos)); }; };
exports.peeks = peeks;
function experiment(F) {
    return function (f) { return function (wa) {
        return function_1.pipe(f(wa.pos), F.map(function (s) { return wa.peek(s); }));
    }; };
}
exports.experiment = experiment;
