"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.traverseReadonlyArrayWithIndex = exports.traverseReadonlyNonEmptyArrayWithIndex = exports.ApT = exports.getRightOnly = exports.getLeftOnly = exports.getRight = exports.getLeft = exports.toTuple2 = exports.exists = exports.elem = exports.Traversable = exports.Foldable = exports.FromThese = exports.fromPredicate = exports.fromOptionK = exports.fromOption = exports.FromEither = exports.getMonad = exports.getChain = exports.getApplicative = exports.getApply = exports.Pointed = exports.flap = exports.Functor = exports.map = exports.Bifunctor = exports.getSemigroup = exports.getEq = exports.getShow = exports.of = exports.traverse = exports.reduceRight = exports.foldMap = exports.reduce = exports.mapLeft = exports.bimap = exports.isBoth = exports.isRight = exports.isLeft = exports.swap = exports.match = exports.matchW = exports.fromOptions = exports.rightOrBoth = exports.leftOrBoth = exports.both = exports.right = exports.left = void 0;
var Bifunctor_1 = require("./Bifunctor");
var Eq_1 = require("./Eq");
var FromEither_1 = require("./FromEither");
var function_1 = require("./function");
var Functor_1 = require("./Functor");
var _ = __importStar(require("./internal"));
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
var left = function (left) { return ({ _tag: 'Left', left: left }); };
exports.left = left;
/**
 * @category constructors
 * @since 3.0.0
 */
var right = function (right) { return ({ _tag: 'Right', right: right }); };
exports.right = right;
/**
 * @category constructors
 * @since 3.0.0
 */
var both = function (left, right) { return ({ _tag: 'Both', left: left, right: right }); };
exports.both = both;
/**
 * @example
 * import { leftOrBoth, left, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(leftOrBoth(() => 'a')(none), left('a'))
 * assert.deepStrictEqual(leftOrBoth(() => 'a')(some(1)), both('a', 1))
 *
 * @category constructors
 * @since 3.0.0
 */
var leftOrBoth = function (e) { return function (ma) {
    return _.isNone(ma) ? exports.left(e()) : exports.both(e(), ma.value);
}; };
exports.leftOrBoth = leftOrBoth;
/**
 * @example
 * import { rightOrBoth, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(rightOrBoth(() => 1)(none), right(1))
 * assert.deepStrictEqual(rightOrBoth(() => 1)(some('a')), both('a', 1))
 *
 * @category constructors
 * @since 3.0.0
 */
var rightOrBoth = function (a) { return function (me) {
    return _.isNone(me) ? exports.right(a()) : exports.both(me.value, a());
}; };
exports.rightOrBoth = rightOrBoth;
/**
 * Takes a pair of `Option`s and attempts to create a `These` from them
 *
 * @example
 * import { fromOptions, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(fromOptions(none, none), none)
 * assert.deepStrictEqual(fromOptions(some('a'), none), some(left('a')))
 * assert.deepStrictEqual(fromOptions(none, some(1)), some(right(1)))
 * assert.deepStrictEqual(fromOptions(some('a'), some(1)), some(both('a', 1)))
 *
 * @category constructors
 * @since 3.0.0
 */
var fromOptions = function (fe, fa) {
    return _.isNone(fe)
        ? _.isNone(fa)
            ? _.none
            : _.some(exports.right(fa.value))
        : _.isNone(fa)
            ? _.some(exports.left(fe.value))
            : _.some(exports.both(fe.value, fa.value));
};
exports.fromOptions = fromOptions;
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
var matchW = function (onLeft, onRight, onBoth) { return function (fa) {
    switch (fa._tag) {
        case 'Left':
            return onLeft(fa.left);
        case 'Right':
            return onRight(fa.right);
        case 'Both':
            return onBoth(fa.left, fa.right);
    }
}; };
exports.matchW = matchW;
/**
 * @category destructors
 * @since 3.0.0
 */
exports.match = exports.matchW;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
exports.swap = exports.match(exports.right, exports.left, function (e, a) { return exports.both(a, e); });
// -------------------------------------------------------------------------------------
// refinements
// -------------------------------------------------------------------------------------
/**
 * Returns `true` if the these is an instance of `Left`, `false` otherwise
 *
 * @category guards
 * @since 3.0.0
 */
var isLeft = function (fa) { return fa._tag === 'Left'; };
exports.isLeft = isLeft;
/**
 * Returns `true` if the these is an instance of `Right`, `false` otherwise
 *
 * @category guards
 * @since 3.0.0
 */
var isRight = function (fa) { return fa._tag === 'Right'; };
exports.isRight = isRight;
/**
 * Returns `true` if the these is an instance of `Both`, `false` otherwise
 *
 * @category guards
 * @since 3.0.0
 */
var isBoth = function (fa) { return fa._tag === 'Both'; };
exports.isBoth = isBoth;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
var bimap = function (f, g) { return function (fa) {
    return exports.isLeft(fa) ? exports.left(f(fa.left)) : exports.isRight(fa) ? exports.right(g(fa.right)) : exports.both(f(fa.left), g(fa.right));
}; };
exports.bimap = bimap;
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
exports.mapLeft = 
/*#__PURE__*/
Bifunctor_1.mapLeftDefault(exports.bimap);
/**
 * @category Foldable
 * @since 3.0.0
 */
var reduce = function (b, f) { return function (fa) { return (exports.isLeft(fa) ? b : f(b, fa.right)); }; };
exports.reduce = reduce;
/**
 * @category Foldable
 * @since 3.0.0
 */
var foldMap = function (M) { return function (f) { return function (fa) { return (exports.isLeft(fa) ? M.empty : f(fa.right)); }; }; };
exports.foldMap = foldMap;
/**
 * @category Foldable
 * @since 3.0.0
 */
var reduceRight = function (b, f) { return function (fa) { return (exports.isLeft(fa) ? b : f(fa.right, b)); }; };
exports.reduceRight = reduceRight;
/**
 * @since 3.0.0
 */
var traverse = function (F) { return function (f) { return function (ta) {
    return exports.isLeft(ta)
        ? F.of(ta)
        : exports.isRight(ta)
            ? function_1.pipe(f(ta.right), F.map(exports.right))
            : function_1.pipe(f(ta.right), F.map(function (b) { return exports.both(ta.left, b); }));
}; }; };
exports.traverse = traverse;
/**
 * @category Pointed
 * @since 3.0.0
 */
exports.of = exports.right;
/**
 * @category instances
 * @since 3.0.0
 */
var getShow = function (SE, SA) { return ({
    show: exports.match(function (l) { return "left(" + SE.show(l) + ")"; }, function (a) { return "right(" + SA.show(a) + ")"; }, function (l, a) { return "both(" + SE.show(l) + ", " + SA.show(a) + ")"; })
}); };
exports.getShow = getShow;
/**
 * @category instances
 * @since 3.0.0
 */
var getEq = function (EE, EA) {
    return Eq_1.fromEquals(function (second) { return function (first) {
        return exports.isLeft(first)
            ? exports.isLeft(second) && EE.equals(second.left)(first.left)
            : exports.isRight(first)
                ? exports.isRight(second) && EA.equals(second.right)(first.right)
                : exports.isBoth(second) && EE.equals(second.left)(first.left) && EA.equals(second.right)(first.right);
    }; });
};
exports.getEq = getEq;
/**
 * @category instances
 * @since 3.0.0
 */
var getSemigroup = function (SE, SA) { return ({
    concat: function (second) { return function (first) {
        return exports.isLeft(first)
            ? exports.isLeft(second)
                ? exports.left(SE.concat(second.left)(first.left))
                : exports.isRight(second)
                    ? exports.both(first.left, second.right)
                    : exports.both(SE.concat(second.left)(first.left), second.right)
            : exports.isRight(first)
                ? exports.isLeft(second)
                    ? exports.both(second.left, first.right)
                    : exports.isRight(second)
                        ? exports.right(SA.concat(second.right)(first.right))
                        : exports.both(second.left, SA.concat(second.right)(first.right))
                : exports.isLeft(second)
                    ? exports.both(SE.concat(second.left)(first.left), first.right)
                    : exports.isRight(second)
                        ? exports.both(first.left, SA.concat(second.right)(first.right))
                        : exports.both(SE.concat(second.left)(first.left), SA.concat(second.right)(first.right));
    }; }
}); };
exports.getSemigroup = getSemigroup;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Bifunctor = {
    bimap: exports.bimap,
    mapLeft: exports.mapLeft
};
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
exports.map = 
/*#__PURE__*/
Bifunctor_1.map(exports.Bifunctor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = 
/*#__PURE__*/
Functor_1.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Pointed = {
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
var getApply = function (S) { return ({
    map: exports.map,
    ap: function (fa) { return function (fab) {
        return exports.isLeft(fab)
            ? exports.isLeft(fa)
                ? exports.left(S.concat(fa.left)(fab.left))
                : exports.isRight(fa)
                    ? exports.left(fab.left)
                    : exports.left(S.concat(fa.left)(fab.left))
            : exports.isRight(fab)
                ? exports.isLeft(fa)
                    ? exports.left(fa.left)
                    : exports.isRight(fa)
                        ? exports.right(fab.right(fa.right))
                        : exports.both(fa.left, fab.right(fa.right))
                : exports.isLeft(fa)
                    ? exports.left(S.concat(fa.left)(fab.left))
                    : exports.isRight(fa)
                        ? exports.both(fab.left, fab.right(fa.right))
                        : exports.both(S.concat(fa.left)(fab.left), fab.right(fa.right));
    }; }
}); };
exports.getApply = getApply;
/**
 * @category instances
 * @since 3.0.0
 */
var getApplicative = function (S) {
    var A = exports.getApply(S);
    return {
        map: exports.map,
        ap: A.ap,
        of: exports.of
    };
};
exports.getApplicative = getApplicative;
/**
 * @category instances
 * @since 3.0.0
 */
var getChain = function (S) {
    var chain = function (f) { return function (ma) {
        if (exports.isLeft(ma)) {
            return ma;
        }
        if (exports.isRight(ma)) {
            return f(ma.right);
        }
        var fb = f(ma.right);
        return exports.isLeft(fb)
            ? exports.left(S.concat(fb.left)(ma.left))
            : exports.isRight(fb)
                ? exports.both(ma.left, fb.right)
                : exports.both(S.concat(fb.left)(ma.left), fb.right);
    }; };
    return {
        map: exports.map,
        chain: chain
    };
};
exports.getChain = getChain;
/**
 * @category instances
 * @since 3.0.0
 */
var getMonad = function (S) {
    var C = exports.getChain(S);
    return {
        map: exports.map,
        of: exports.of,
        chain: C.chain
    };
};
exports.getMonad = getMonad;
/**
 * @category instances
 * @since 3.0.0
 */
exports.FromEither = {
    fromEither: function_1.identity
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
exports.fromOption = 
/*#__PURE__*/
FromEither_1.fromOption(exports.FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
exports.fromOptionK = 
/*#__PURE__*/
FromEither_1.fromOptionK(exports.FromEither);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
exports.fromPredicate = 
/*#__PURE__*/
FromEither_1.fromPredicate(exports.FromEither);
/**
 * @category instances
 * @since 3.0.0
 */
exports.FromThese = {
    fromThese: function_1.identity
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Foldable = {
    reduce: exports.reduce,
    foldMap: exports.foldMap,
    reduceRight: exports.reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Traversable = {
    map: exports.map,
    traverse: exports.traverse
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
var elem = function (E) { return function (a) { return exports.exists(E.equals(a)); }; };
exports.elem = elem;
/**
 * @since 3.0.0
 */
var exists = function (predicate) { return function (ma) {
    return exports.isLeft(ma) ? false : predicate(ma.right);
}; };
exports.exists = exists;
/**
 * @example
 * import { toTuple2, left, right, both } from 'fp-ts/These'
 *
 * const f = toTuple2(() => 'a', () => 1)
 * assert.deepStrictEqual(f(left('b')), ['b', 1])
 * assert.deepStrictEqual(f(right(2)), ['a', 2])
 * assert.deepStrictEqual(f(both('b', 2)), ['b', 2])
 *
 * @since 3.0.0
 */
var toTuple2 = function (e, a) { return function (fa) {
    return exports.isLeft(fa) ? [fa.left, a()] : exports.isRight(fa) ? [e(), fa.right] : [fa.left, fa.right];
}; };
exports.toTuple2 = toTuple2;
/**
 * Returns an `E` value if possible
 *
 * @example
 * import { getLeft, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getLeft(left('a')), some('a'))
 * assert.deepStrictEqual(getLeft(right(1)), none)
 * assert.deepStrictEqual(getLeft(both('a', 1)), some('a'))
 *
 * @since 3.0.0
 */
var getLeft = function (fa) {
    return exports.isLeft(fa) ? _.some(fa.left) : exports.isRight(fa) ? _.none : _.some(fa.left);
};
exports.getLeft = getLeft;
/**
 * Returns an `A` value if possible
 *
 * @example
 * import { getRight, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getRight(left('a')), none)
 * assert.deepStrictEqual(getRight(right(1)), some(1))
 * assert.deepStrictEqual(getRight(both('a', 1)), some(1))
 *
 * @since 3.0.0
 */
var getRight = function (fa) {
    return exports.isLeft(fa) ? _.none : exports.isRight(fa) ? _.some(fa.right) : _.some(fa.right);
};
exports.getRight = getRight;
/**
 * Returns the `E` value if and only if the value is constructed with `Left`
 *
 * @example
 * import { getLeftOnly, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getLeftOnly(left('a')), some('a'))
 * assert.deepStrictEqual(getLeftOnly(right(1)), none)
 * assert.deepStrictEqual(getLeftOnly(both('a', 1)), none)
 *
 * @since 3.0.0
 */
var getLeftOnly = function (fa) { return (exports.isLeft(fa) ? _.some(fa.left) : _.none); };
exports.getLeftOnly = getLeftOnly;
/**
 * Returns the `A` value if and only if the value is constructed with `Right`
 *
 * @example
 * import { getRightOnly, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getRightOnly(left('a')), none)
 * assert.deepStrictEqual(getRightOnly(right(1)), some(1))
 * assert.deepStrictEqual(getRightOnly(both('a', 1)), none)
 *
 * @since 3.0.0
 */
var getRightOnly = function (fa) { return (exports.isRight(fa) ? _.some(fa.right) : _.none); };
exports.getRightOnly = getRightOnly;
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.ApT = 
/*#__PURE__*/
exports.of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(S))`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArrayWithIndex = function (S) { return function (f) { return function (as) {
    var e = _.none;
    var t = f(0, _.head(as));
    if (exports.isLeft(t)) {
        return t;
    }
    if (exports.isBoth(t)) {
        e = _.some(t.left);
    }
    var out = [t.right];
    for (var i = 1; i < as.length; i++) {
        var t_1 = f(i, as[i]);
        if (exports.isLeft(t_1)) {
            return t_1;
        }
        if (exports.isBoth(t_1)) {
            e = _.isNone(e) ? _.some(t_1.left) : _.some(S.concat(t_1.left)(e.value));
        }
        out.push(t_1.right);
    }
    return _.isNone(e) ? exports.right(out) : exports.both(e.value, out);
}; }; };
exports.traverseReadonlyNonEmptyArrayWithIndex = traverseReadonlyNonEmptyArrayWithIndex;
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(S))`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArrayWithIndex = function (S) { return function (f) {
    var g = exports.traverseReadonlyNonEmptyArrayWithIndex(S)(f);
    return function (as) { return (_.isNonEmpty(as) ? g(as) : exports.ApT); };
}; };
exports.traverseReadonlyArrayWithIndex = traverseReadonlyArrayWithIndex;
