"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.censor = exports.listens = exports.listen = exports.tracks = exports.getComonad = exports.flap = exports.Functor = exports.map = void 0;
var Functor_1 = require("./Functor");
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
var map = function (f) { return function (fa) { return function (p) { return f(fa(p)); }; }; };
exports.map = map;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = 
/*#__PURE__*/
Functor_1.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
var getComonad = function (monoid) { return ({
    map: exports.map,
    extend: function (f) { return function (wa) { return function (p1) { return f(function (p2) { return wa(monoid.concat(p2)(p1)); }); }; }; },
    extract: function (wa) { return wa(monoid.empty); }
}); };
exports.getComonad = getComonad;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Extracts a value at a relative position which depends on the current value.
 *
 * @since 3.0.0
 */
var tracks = function (M) { return function (f) { return function (wa) { return wa(f(wa(M.empty))); }; }; };
exports.tracks = tracks;
/**
 * Get the current position.
 *
 * @since 3.0.0
 */
var listen = function (wa) { return function (p) { return [wa(p), p]; }; };
exports.listen = listen;
/**
 * Get a value which depends on the current position.
 *
 * @since 3.0.0
 */
var listens = function (f) { return function (wa) { return function (p) { return [
    wa(p),
    f(p)
]; }; }; };
exports.listens = listens;
/**
 * Apply a function to the current position.
 *
 * @since 3.0.0
 */
var censor = function (f) { return function (wa) { return function (p) { return wa(f(p)); }; }; };
exports.censor = censor;
