"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute = exports.evaluate = exports.flap = exports.Functor = exports.getMonad = exports.getChain = exports.getApplicative = exports.getApply = exports.getPointed = exports.map = exports.censor = exports.listens = exports.pass = exports.listen = exports.tell = void 0;
var Functor_1 = require("./Functor");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
var tell = function (w) { return function () { return [undefined, w]; }; };
exports.tell = tell;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
var listen = function (fa) { return function () {
    var _a = fa(), a = _a[0], w = _a[1];
    return [[a, w], w];
}; };
exports.listen = listen;
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
var pass = function (fa) { return function () {
    var _a = fa(), _b = _a[0], a = _b[0], f = _b[1], w = _a[1];
    return [a, f(w)];
}; };
exports.pass = pass;
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
var listens = function (f) { return function (fa) { return function () {
    var _a = fa(), a = _a[0], w = _a[1];
    return [[a, f(w)], w];
}; }; };
exports.listens = listens;
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
var censor = function (f) { return function (fa) { return function () {
    var _a = fa(), a = _a[0], w = _a[1];
    return [a, f(w)];
}; }; };
exports.censor = censor;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
var map = function (f) { return function (fa) { return function () {
    var _a = fa(), a = _a[0], w = _a[1];
    return [f(a), w];
}; }; };
exports.map = map;
/**
 * @category instances
 * @since 3.0.0
 */
var getPointed = function (M) { return ({
    of: function (a) { return function () { return [a, M.empty]; }; }
}); };
exports.getPointed = getPointed;
/**
 * @category instances
 * @since 3.0.0
 */
var getApply = function (S) { return ({
    map: exports.map,
    ap: function (fa) { return function (fab) { return function () {
        var _a = fab(), f = _a[0], w1 = _a[1];
        var _b = fa(), a = _b[0], w2 = _b[1];
        return [f(a), S.concat(w2)(w1)];
    }; }; }
}); };
exports.getApply = getApply;
/**
 * @category instances
 * @since 3.0.0
 */
var getApplicative = function (M) {
    var A = exports.getApply(M);
    var P = exports.getPointed(M);
    return {
        map: exports.map,
        ap: A.ap,
        of: P.of
    };
};
exports.getApplicative = getApplicative;
/**
 * @category instances
 * @since 3.0.0
 */
var getChain = function (S) {
    return {
        map: exports.map,
        chain: function (f) { return function (ma) { return function () {
            var _a = ma(), a = _a[0], w1 = _a[1];
            var _b = f(a)(), b = _b[0], w2 = _b[1];
            return [b, S.concat(w2)(w1)];
        }; }; }
    };
};
exports.getChain = getChain;
/**
 * @category instances
 * @since 3.0.0
 */
var getMonad = function (M) {
    var P = exports.getPointed(M);
    var C = exports.getChain(M);
    return {
        map: exports.map,
        of: P.of,
        chain: C.chain
    };
};
exports.getMonad = getMonad;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = 
/*#__PURE__*/
Functor_1.flap(exports.Functor);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
var evaluate = function (fa) { return fa()[0]; };
exports.evaluate = evaluate;
/**
 * @since 3.0.0
 */
var execute = function (fa) { return fa()[1]; };
exports.execute = execute;
