"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAssignSemigroup = void 0;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * Return a semigroup which works like `Object.assign`.
 *
 * @example
 * import { getAssignSemigroup } from 'fp-ts/struct'
 * import { pipe } from 'fp-ts/function'
 *
 * interface Person {
 *   readonly name: string
 *   readonly age: number
 * }
 *
 * const S = getAssignSemigroup<Person>()
 * assert.deepStrictEqual(pipe({ name: 'name', age: 23 }, S.concat({ name: 'name', age: 24 })), { name: 'name', age: 24 })
 *
 * @category instances
 * @since 3.0.0
 */
var getAssignSemigroup = function () { return ({
    concat: function (second) { return function (first) { return Object.assign({}, first, second); }; }
}); };
exports.getAssignSemigroup = getAssignSemigroup;
