/**
 * The `Apply` class provides the `ap` which is used to apply a function to an argument under a type constructor.
 *
 * `Apply` can be used to lift functions of two or more arguments to work on values wrapped with the type constructor
 * `f`.
 *
 * Instances must satisfy the following law in addition to the `Functor` laws:
 *
 * 1. Associative composition: `fbc |> map(bc => ab => a => bc(ab(a))) |> ap(fab) <-> fbc |> ap(fab |> ap(fa))`
 *
 * Formally, `Apply` represents a strong lax semi-monoidal endofunctor.
 *
 * @since 3.0.0
 */
import { flow, pipe } from './function';
import { reverse } from './Semigroup';
export function ap(F, G) {
    return (fa) => flow(F.map((gab) => (ga) => G.ap(ga)(gab)), F.ap(fa));
}
export function apFirst(A) {
    return (second) => flow(A.map((a) => () => a), A.ap(second));
}
export function apSecond(A) {
    return (second) => flow(A.map(() => (b) => b), A.ap(second));
}
export function apS(F) {
    return (name, fb) => flow(F.map((a) => (b) => Object.assign({}, a, { [name]: b })), F.ap(fb));
}
export function apT(F) {
    return (fb) => (fas) => pipe(fas, F.map((a) => (b) => [...a, b]), F.ap(fb));
}
export function getApplySemigroup(F) {
    return (S) => {
        const f = reverse(S).concat;
        return {
            concat: (second) => (first) => pipe(first, F.map(f), F.ap(second))
        };
    };
}
