/**
 * The `Bounded` type class represents totally ordered types that have an upper and lower boundary.
 *
 * Instances should satisfy the following law in addition to the `Ord` laws:
 *
 * - Bounded: `bottom <= a <= top`
 *
 * @since 3.0.0
 */
import * as O from './Ord';
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Clamp a value between `bottom` and `top` values.
 *
 * @category utils
 * @since 3.0.0
 */
export const clamp = (B) => O.clamp(B)(B.bottom, B.top);
/**
 * Reverses the `Ord` of a `Bounded` and swaps `top` and `bottom` values.
 *
 * @category utils
 * @since 3.0.0
 */
export const reverse = (B) => {
    const R = O.reverse(B);
    return {
        compare: R.compare,
        top: B.bottom,
        bottom: B.top
    };
};
