/**
 * `Compactable` represents data structures which can be _compacted_/_separated_.
 *
 * @since 3.0.0
 */
import { fromOption } from './Either';
import { constVoid, flow, pipe } from './function';
import { map } from './Functor';
import { getLeft, getRight } from './Option';
import { right, separated } from './Separated';
export function compactDefault(F) {
    const fromOptionF = F.map(fromOption(constVoid));
    return (separate) => flow(fromOptionF, separate, right);
}
export function separateDefault(F) {
    return (compact) => (fe) => separated(pipe(fe, F.map(getLeft), compact), pipe(fe, F.map(getRight), compact));
}
export function compact(F, G) {
    return F.map(G.compact);
}
export function separate(F, C, G) {
    const _compact = compact(F, C);
    const _map = map(F, G);
    return (fge) => separated(pipe(fge, _map(getLeft), _compact), pipe(fge, _map(getRight), _compact));
}
