import { map as map_, mapLeftDefault } from './Bifunctor';
import { flow, identity, unsafeCoerce } from './function';
import { flap as flap_ } from './Functor';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const make = unsafeCoerce;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Contravariant
 * @since 3.0.0
 */
export const contramap = () => unsafeCoerce;
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = (f) => flow(f, make);
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
mapLeftDefault(bimap);
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => ({
    show: (c) => `make(${S.show(c)})`
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getOrd = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getBounded = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonoid = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemiring = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getRing = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getHeytingAlgebra = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getBooleanAlgebra = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const Contravariant = {
    contramap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
map_(Bifunctor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map,
    ap: (fa) => (fab) => make(S.concat(fa)(fab))
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (M) => {
    const A = getApply(M);
    return {
        map: A.map,
        ap: A.ap,
        of: () => make(M.empty)
    };
};
