import { pipe } from './function';
import * as O from './Ord';
import * as N from './number';
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Eq = {
    equals: (second) => (first) => first.valueOf() === second.valueOf()
};
/**
 * @category instances
 * @since 3.0.0
 */
export const EqDate = {
    equals: (second) => (first) => first.getDate() === second.getDate()
};
/**
 * @category instances
 * @since 3.0.0
 */
export const EqMonth = {
    equals: (second) => (first) => first.getMonth() === second.getMonth()
};
/**
 * @category instances
 * @since 3.0.0
 */
export const EqYear = {
    equals: (second) => (first) => first.getFullYear() === second.getFullYear()
};
/**
 * @example
 * import { Ord } from 'fp-ts/Date'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(new Date(1, 1, 2020), Ord.compare(new Date(1, 1, 2021))), -1)
 *
 * @category instances
 * @since 3.0.0
 */
export const Ord = 
/*#__PURE__*/
pipe(N.Ord, 
/*#__PURE__*/
O.contramap((date) => date.valueOf()));
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Returns the current `Date`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const create = () => new Date();
/**
 * Returns the number of milliseconds elapsed since January 1, 1970, 00:00:00 UTC.
 *
 * @since 3.0.0
 */
export const now = () => new Date().getTime();
