/**
 * @since 3.0.0
 */
import { ap as ap_ } from './Apply';
import * as E from './Either';
import { flow, pipe } from './function';
import { map as map_ } from './Functor';
export function right(F) {
    return flow(E.right, F.of);
}
export function left(F) {
    return flow(E.left, F.of);
}
export function rightF(F) {
    return F.map(E.right);
}
export function leftF(F) {
    return F.map(E.left);
}
export function fromNullable(F) {
    return (e) => flow(E.fromNullable(() => e), F.of);
}
export function fromNullableK(F) {
    const fromNullableF = fromNullable(F);
    return (e) => {
        const fromNullableFE = fromNullableF(e);
        return (f) => flow(f, fromNullableFE);
    };
}
export function chainNullableK(M) {
    const chainM = chain(M);
    const fromNullableKM = fromNullableK(M);
    return (e) => {
        const fromNullableKMe = fromNullableKM(e);
        return (f) => chainM(fromNullableKMe(f));
    };
}
export function map(F) {
    return map_(F, E.Functor);
}
export function ap(F) {
    return ap_(F, E.Apply);
}
export function chain(M) {
    return (f) => M.chain((e) => (E.isLeft(e) ? M.of(e) : f(e.right)));
}
export function alt(M) {
    return (second) => M.chain((e) => (E.isLeft(e) ? second() : M.of(e)));
}
export function bimap(F) {
    return flow(E.bimap, F.map);
}
export function mapLeft(F) {
    return (f) => F.map(E.mapLeft(f));
}
export function altValidation(M, S) {
    const rightM = right(M);
    return (second) => (first) => pipe(first, M.chain(E.match((e1) => pipe(second(), M.map(E.mapLeft((e2) => S.concat(e2)(e1)))), rightM)));
}
export function match(F) {
    return flow(E.match, F.map);
}
export function matchE(M) {
    return flow(E.match, M.chain);
}
export function getOrElse(F) {
    return flow(E.getOrElse, F.map);
}
export function getOrElseE(M) {
    return (onLeft) => M.chain(E.match(onLeft, M.of));
}
export function orElse(M) {
    return (onLeft) => M.chain((e) => (E.isLeft(e) ? onLeft(e.left) : M.of(e)));
}
export function orLeft(M) {
    return (onLeft) => M.chain(E.match((e) => pipe(onLeft(e), M.map(E.left)), (a) => M.of(E.right(a))));
}
export function orElseFirst(M) {
    const orElseM = orElse(M);
    return (onLeft) => orElseM((e) => pipe(onLeft(e), M.map((eb) => (E.isLeft(eb) ? eb : E.left(e)))));
}
export function swap(F) {
    return F.map(E.swap);
}
export function toUnion(F) {
    return F.map(E.toUnion);
}
export function bracket(M) {
    const leftM = left(M);
    return (acquire, use, release) => pipe(acquire, M.chain(E.match(leftM, (a) => pipe(use(a), M.chain((e) => pipe(release(a, e), M.chain(E.match(leftM, () => M.of(e)))))))));
}
