/**
 * A `Foldable` with an additional index.
 *
 * A `FoldableWithIndex` instance must be compatible with its `Foldable` instance:
 *
 * ```ts
 * reduce(b, f) = reduceWithIndex(b, (_, b, a) => f(b, a))
 * foldMap(M)(f) = foldMapWithIndex(M)((_, a) => f(a))
 * reduceRight(b, f) = reduceRightWithIndex(b, (_, a, b) => f(a, b))
 * ```
 *
 * @since 3.0.0
 */
import { pipe } from './function';
export function reduceWithIndex(F, G) {
    return (b, f) => F.reduceWithIndex(b, (i, b, ga) => pipe(ga, G.reduceWithIndex(b, (j, b, a) => f([i, j], b, a))));
}
export function foldMapWithIndex(F, G) {
    return (M) => {
        const foldMapWithIndexF = F.foldMapWithIndex(M);
        const foldMapWithIndexG = G.foldMapWithIndex(M);
        return (f) => foldMapWithIndexF((i, ga) => pipe(ga, foldMapWithIndexG((j, a) => f([i, j], a))));
    };
}
export function reduceRightWithIndex(F, G) {
    return (b, f) => F.reduceRightWithIndex(b, (i, ga, b) => pipe(ga, G.reduceRightWithIndex(b, (j, a, b) => f([i, j], a, b))));
}
