/**
 * The `FromEither` type class represents those data types which support errors.
 *
 * @since 3.0.0
 */
import { chainFirst } from './Chain';
import { flow, pipe } from './function';
import * as _ from './internal';
export function fromOption(F) {
    return (onNone) => (ma) => F.fromEither(_.isNone(ma) ? _.left(onNone()) : _.right(ma.value));
}
export function fromPredicate(F) {
    return (predicate) => (a) => F.fromEither(predicate(a) ? _.right(a) : _.left(a));
}
export function fromOptionK(F) {
    const fromOptionF = fromOption(F);
    return (onNone) => {
        const from = fromOptionF(onNone);
        return (f) => flow(f, from);
    };
}
export function chainOptionK(F, M) {
    const fromOptionKF = fromOptionK(F);
    return (onNone) => flow(fromOptionKF(onNone), M.chain);
}
export function fromEitherK(F) {
    return (f) => flow(f, F.fromEither);
}
export function chainEitherK(F, M) {
    return flow(fromEitherK(F), M.chain);
}
export function chainFirstEitherK(F, M) {
    return flow(fromEitherK(F), chainFirst(M));
}
export function filterOrElse(F, M) {
    return (predicate, onFalse) => (ma) => pipe(ma, M.chain((a) => F.fromEither(predicate(a) ? _.right(a) : _.left(onFalse(a)))));
}
