import { ap as ap_, apFirst as apFirst_, apS as apS_, apSecond as apSecond_, apT as apT_ } from './Apply';
import { ap as apSeq_, bind as bind_, chainFirst as chainFirst_ } from './Chain';
import { compact as compact_, separate as separate_ } from './Compactable';
import * as E from './Either';
import * as ET from './EitherT';
import { filter, filterMap, partition, partitionMap } from './Filterable';
import { chainEitherK as chainEitherK_, chainOptionK as chainOptionK_, filterOrElse as filterOrElse_, fromEitherK as fromEitherK_, fromOption as fromOption_, fromOptionK as fromOptionK_, fromPredicate as fromPredicate_, chainFirstEitherK as chainFirstEitherK_ } from './FromEither';
import { chainFirstIOK as chainFirstIOK_, chainIOK as chainIOK_, fromIOK as fromIOK_ } from './FromIO';
import { flow, identity, pipe, SK } from './function';
import { bindTo as bindTo_, flap as flap_, tupled as tupled_ } from './Functor';
import * as _ from './internal';
import * as I from './IO';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = 
/*#__PURE__*/
ET.left(I.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = 
/*#__PURE__*/
ET.right(I.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = 
/*#__PURE__*/
ET.rightF(I.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = 
/*#__PURE__*/
ET.leftF(I.Functor);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = I.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = rightIO;
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = 
/*#__PURE__*/
ET.match(I.Functor);
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchW = match;
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchE = 
/*#__PURE__*/
ET.matchE(I.Monad);
/**
 * Less strict version of [`matchE`](#matchE).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchEW = matchE;
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/
ET.getOrElse(I.Functor);
/**
 * Less strict version of [`getOrElse`](#getOrElse).
 *
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseW = getOrElse;
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = 
/*#__PURE__*/
ET.getOrElseE(I.Monad);
/**
 * Less strict version of [`getOrElseE`](#getOrElseE).
 *
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseEW = getOrElseE;
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * Constructs a new `IOEither` from a function that performs a side effect and might throw.
 *
 * See also [`tryCatchK`](#tryCatchK).
 *
 * @category interop
 * @since 3.0.0
 */
export const tryCatch = (f) => () => E.tryCatch(f);
/**
 * Converts a function that may throw to one returning a `IOEither`.
 *
 * @category interop
 * @since 3.0.0
 */
export const tryCatchK = (f, onThrow) => (...a) => pipe(tryCatch(() => f(...a)), mapLeft(onThrow));
/**
 * @category interop
 * @since 3.0.0
 */
export const toUnion = 
/*#__PURE__*/
ET.toUnion(I.Functor);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElse = 
/*#__PURE__*/
ET.orElse(I.Monad);
/**
 * Less strict version of [`orElse`](#orElse).
 *
 * @category combinators
 * @since 3.0.0
 */
export const orElseW = orElse;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElseFirst = 
/*#__PURE__*/
ET.orElseFirst(I.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElseFirstW = orElseFirst;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElseFirstIOK = (onLeft) => orElseFirst(fromIOK(onLeft));
/**
 * @category combinators
 * @since 3.0.0
 */
export const orLeft = 
/*#__PURE__*/
ET.orLeft(I.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = 
/*#__PURE__*/
ET.swap(I.Functor);
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
ET.map(I.Functor);
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = 
/*#__PURE__*/
ET.bimap(I.Functor);
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
ET.mapLeft(I.Functor);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/
ET.ap(I.Apply);
/**
 * Less strict version of [`ap`](#ap).
 *
 * @category Apply
 * @since 3.0.0
 */
export const apW = ap;
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export const chain = 
/*#__PURE__*/
ET.chain(I.Monad);
/**
 * Less strict version of [`chain`](#chain).
 *
 * @category Chain
 * @since 3.0.0
 */
export const chainW = chain;
/**
 * Less strict version of [`flatten`](#flatten).
 *
 * @category combinators
 * @since 3.0.0
 */
export const flattenW = 
/*#__PURE__*/
chainW(identity);
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const flatten = flattenW;
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export const alt = 
/*#__PURE__*/
ET.alt(I.Monad);
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export const altW = alt;
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicativeIOValidation = (S) => ({
    map,
    ap: ap_(I.Apply, E.getApplicativeValidation(S)),
    of
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getAltIOValidation = (S) => {
    return {
        map,
        alt: ET.altValidation(I.Monad, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getCompactable = (M) => {
    const C = E.getCompactable(M);
    return {
        compact: compact_(I.Functor, C),
        separate: separate_(I.Functor, C, E.Functor)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFilterable = (M) => {
    const F = E.getFilterable(M);
    return {
        filter: filter(I.Functor, F),
        filterMap: filterMap(I.Functor, F),
        partition: partition(I.Functor, F),
        partitionMap: partitionMap(I.Functor, F)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplyPar = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apFirst = 
/*#__PURE__*/
apFirst_(ApplyPar);
/**
 * Less strict version of [`apFirst`](#apfirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export const apFirstW = apFirst;
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apSecond = 
/*#__PURE__*/
apSecond_(ApplyPar);
/**
 * Less strict version of [`apSecond`](#apsecond).
 *
 * @category combinators
 * @since 3.0.0
 */
export const apSecondW = apSecond;
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativePar = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    map,
    chain
};
const apSeq = 
/*#__PURE__*/
apSeq_(Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplySeq = {
    map,
    ap: apSeq
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativeSeq = {
    map,
    ap: apSeq,
    of
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const chainFirst = 
/*#__PURE__*/
chainFirst_(Chain);
/**
 * Less strict version of [`chainFirst`](#chainFirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstW = chainFirst;
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Alt = {
    map,
    alt
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/
fromIOK_(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainIOK = 
/*#__PURE__*/
chainIOK_(FromIO, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstIOK = 
/*#__PURE__*/
chainFirstIOK_(FromIO, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/
fromOption_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/
fromOptionK_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainOptionK = 
/*#__PURE__*/
chainOptionK_(FromEither, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherK = 
/*#__PURE__*/
chainEitherK_(FromEither, Chain);
/**
 * Less strict version of [`chainEitherK`](#chainEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherKW = chainEitherK;
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherK = 
/*#__PURE__*/
chainFirstEitherK_(FromEither, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherKW = chainFirstEitherK;
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/
fromPredicate_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElse = 
/*#__PURE__*/
filterOrElse_(FromEither, Chain);
/**
 * Less strict version of [`filterOrElse`](#filterOrElse).
 *
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElseW = filterOrElse;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = 
/*#__PURE__*/
fromEitherK_(FromEither);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Make sure that a resource is cleaned up in the event of an exception (\*). The release action is called regardless of
 * whether the body action throws (\*) or returns.
 *
 * (\*) i.e. returns a `Left`
 *
 * @since 3.0.0
 */
export const bracket = 
/*#__PURE__*/
ET.bracket(I.Monad);
/**
 * Less strict version of [`bracket`](#bracket).
 *
 * @since 3.0.0
 */
export const bracketW = bracket;
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = 
/*#__PURE__*/
of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/
bindTo_(Functor);
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/
bind_(Chain);
/**
 * Less strict version of [`bind`](#bind).
 *
 * @since 3.0.0
 */
export const bindW = bind;
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apS = 
/*#__PURE__*/
apS_(ApplyPar);
/**
 * Less strict version of [`apS`](#apS).
 *
 * @since 3.0.0
 */
export const apSW = apS;
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/
tupled_(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/
apT_(ApplyPar);
/**
 * Less strict version of [`apT`](#apT).
 *
 * @since 3.0.0
 */
export const apTW = apT;
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(I.traverseReadonlyNonEmptyArrayWithIndex(f), I.map(E.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (f) => (as) => () => {
    const e = f(0, _.head(as))();
    if (_.isLeft(e)) {
        return e;
    }
    const out = [e.right];
    for (let i = 1; i < as.length; i++) {
        const e = f(i, as[i])();
        if (_.isLeft(e)) {
            return e;
        }
        out.push(e.right);
    }
    return _.right(out);
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
