import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_ } from './Apply';
import { bind as bind_, chainFirst as chainFirst_ } from './Chain';
import { compact as compact_, separate as separate_ } from './Compactable';
import { filter as filter_, filterMap as filterMap_, partition as partition_, partitionMap as partitionMap_ } from './Filterable';
import { chainEitherK as chainEitherK_, chainFirstEitherK as chainFirstEitherK_, fromEitherK as fromEitherK_ } from './FromEither';
import { chainFirstIOK as chainFirstIOK_, chainIOK as chainIOK_, fromIOK as fromIOK_ } from './FromIO';
import { flow, identity, SK } from './function';
import { bindTo as bindTo_, flap as flap_ } from './Functor';
import * as _ from './internal';
import * as O from './Option';
import * as OT from './OptionT';
import * as I from './IO';
import { guard as guard_ } from './Zero';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const some = 
/*#__PURE__*/
OT.some(I.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/
OT.fromPredicate(I.Pointed);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = I.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = 
/*#__PURE__*/
OT.fromEither(I.Pointed);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = 
/*#__PURE__*/
OT.fromF(I.Functor);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = 
/*#__PURE__*/
I.map(O.fromEither);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = 
/*#__PURE__*/
OT.match(I.Functor);
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchW = match;
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchE = 
/*#__PURE__*/
OT.matchE(I.Chain);
/**
 * Alias of [`matchE`](#matche).
 *
 * @category destructors
 * @since 3.0.0
 */
export const fold = matchE;
/**
 * Less strict version of [`matchE`](#matche).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchEW = matchE;
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/
OT.getOrElse(I.Functor);
/**
 * Less strict version of [`getOrElse`](#getorelse).
 *
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseW = getOrElse;
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = 
/*#__PURE__*/
OT.getOrElseE(I.Monad);
/**
 * Less strict version of [`getOrElseE`](#getOrElseE).
 *
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseEW = getOrElseE;
/**
 * @category destructors
 * @since 3.0.0
 */
export const toUndefined = I.map(O.toUndefined);
/**
 * @category destructors
 * @since 3.0.0
 */
export const toNullable = I.map(O.toNullable);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/
OT.fromNullable(I.Pointed);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = 
/*#__PURE__*/
OT.fromNullableK(I.Pointed);
/**
 * @category interop
 * @since 3.0.0
 */
export const chainNullableK = 
/*#__PURE__*/
OT.chainNullableK(I.Monad);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/
OT.fromOptionK(I.Pointed);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainOptionK = 
/*#__PURE__*/
OT.chainOptionK(I.Monad);
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
OT.map(I.Functor);
/**
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/
OT.ap(I.Apply);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = some;
/**
 * @category Monad
 * @since 3.0.0
 */
export const chain = 
/*#__PURE__*/
OT.chain(I.Monad);
/**
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = 
/*#__PURE__*/
chain(identity);
/**
 * @category Alt
 * @since 3.0.0
 */
export const alt = 
/*#__PURE__*/
OT.alt(I.Monad);
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export const altW = alt;
/**
 * @category Zero
 * @since 3.0.0
 */
export const zero = 
/*#__PURE__*/
OT.zero(I.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const none = 
/*#__PURE__*/
zero();
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = 
/*#__PURE__*/
compact_(I.Functor, O.Compactable);
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = 
/*#__PURE__*/
separate_(I.Functor, O.Compactable, O.Functor);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filter = 
/*#__PURE__*/
filter_(I.Functor, O.Filterable);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = 
/*#__PURE__*/
filterMap_(I.Functor, O.Filterable);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partition = 
/*#__PURE__*/
partition_(I.Functor, O.Filterable);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = 
/*#__PURE__*/
partitionMap_(I.Functor, O.Filterable);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
const URI = 'IOOption';
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    URI,
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    URI,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    URI,
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const apFirst = 
/*#__PURE__*/
apFirst_(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const apSecond = 
/*#__PURE__*/
apSecond_(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    URI,
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    URI,
    map,
    chain
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirst = 
/*#__PURE__*/
chainFirst_(Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const Alt = {
    URI,
    map,
    alt
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Zero = {
    URI,
    zero
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const guard = 
/*#__PURE__*/
guard_(Zero, Pointed);
/**
 * @category instances
 * @since 3.0.0
 */
export const Alternative = {
    URI,
    map,
    alt,
    zero
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    URI,
    map,
    of,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Compactable = {
    URI,
    compact,
    separate
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Filterable = {
    URI,
    filter,
    filterMap,
    partition,
    partitionMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    URI,
    fromIO
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/
fromIOK_(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainIOK = 
/*#__PURE__*/
chainIOK_(FromIO, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstIOK = 
/*#__PURE__*/
chainFirstIOK_(FromIO, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    URI,
    fromEither
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = 
/*#__PURE__*/
fromEitherK_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherK = 
/*#__PURE__*/
chainEitherK_(FromEither, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherK = 
/*#__PURE__*/
chainFirstEitherK_(FromEither, Chain);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = 
/*#__PURE__*/
of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/
bindTo_(Functor);
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/
bind_(Chain);
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apS = 
/*#__PURE__*/
apS_(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(I.traverseReadonlyNonEmptyArrayWithIndex(f), I.map(O.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
