/**
 * @since 3.0.0
 */
import { tryCatch } from './Either';
/**
 * Converts a JavaScript Object Notation (JSON) string into an object.
 *
 * @example
 * import * as J from 'fp-ts/Json'
 * import * as E from 'fp-ts/Either'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe('{"a":1}', J.parse), E.right({ a: 1 }))
 * assert.deepStrictEqual(pipe('{"a":}', J.parse), E.left(new SyntaxError('Unexpected token } in JSON at position 5')))
 *
 * @since 3.0.0
 */
export const parse = (s) => tryCatch(() => JSON.parse(s));
/**
 * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.
 *
 * @since 3.0.0
 */
export const stringify = (a) => tryCatch(() => {
    const s = JSON.stringify(a);
    if (typeof s !== 'string') {
        throw new Error('Converting unsupported structure to JSON');
    }
    return s;
});
