/**
 * @since 3.0.0
 */
import * as E from './Eq';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const reverse = (o) => (o === -1 ? 1 : o === 1 ? -1 : 0);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchW = (onLessThan, onEqual, onGreaterThan) => (o) => (o === -1 ? onLessThan() : o === 0 ? onEqual() : onGreaterThan());
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = matchW;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Eq = E.EqStrict;
/**
 * @category instances
 * @since 3.0.0
 */
export const Semigroup = {
    concat: (second) => (first) => (first !== 0 ? first : second)
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monoid = {
    concat: Semigroup.concat,
    empty: 0
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const sign = (n) => (n <= -1 ? -1 : n >= 1 ? 1 : 0);
