import { ap as ap_, apFirst as apFirst_, apS as apS_, apSecond as apSecond_, apT as apT_ } from './Apply';
import { ap as apSeq_, bind as bind_, chainFirst as chainFirst_ } from './Chain';
import { compact as compact_, separate as separate_ } from './Compactable';
import * as E from './Either';
import * as ET from './EitherT';
import { filter, filterMap, partition, partitionMap } from './Filterable';
import { chainEitherK as chainEitherK_, chainOptionK as chainOptionK_, filterOrElse as filterOrElse_, fromEitherK as fromEitherK_, fromOption as fromOption_, fromOptionK as fromOptionK_, fromPredicate as fromPredicate_ } from './FromEither';
import { chainFirstIOK as chainFirstIOK_, chainIOK as chainIOK_, fromIOK as fromIOK_ } from './FromIO';
import { ask as ask_, asks as asks_, chainFirstReaderK as chainFirstReaderK_, chainReaderK as chainReaderK_, fromReaderK as fromReaderK_ } from './FromReader';
import { chainFirstTaskK as chainFirstTaskK_, chainTaskK as chainTaskK_, fromTaskK as fromTaskK_ } from './FromTask';
import { flow, identity, SK } from './function';
import { bindTo as bindTo_, flap as flap_, tupled as tupled_ } from './Functor';
import * as _ from './internal';
import * as R from './Reader';
import * as RT from './ReaderTask';
import * as TE from './TaskEither';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTaskEither = 
/*#__PURE__*/
R.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = 
/*#__PURE__*/
ET.left(RT.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = 
/*#__PURE__*/
ET.right(RT.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = 
/*#__PURE__*/
flow(TE.rightTask, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = 
/*#__PURE__*/
flow(TE.leftTask, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReader = (ma) => flow(ma, TE.right);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReader = (me) => flow(me, TE.left);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReaderTask = 
/*#__PURE__*/
ET.rightF(RT.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReaderTask = 
/*#__PURE__*/
ET.leftF(RT.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = 
/*#__PURE__*/
flow(TE.rightIO, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = 
/*#__PURE__*/
flow(TE.leftIO, fromTaskEither);
/**
 * Less strict version of [`asksReaderTaskEitherK`](#asksreadertaskeitherk).
 *
 * @category constructors
 * @since 3.0.0
 */
export const asksReaderTaskEitherW = R.asksReaderW;
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksReaderTaskEither = asksReaderTaskEitherW;
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = RT.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReader = rightReader;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTask = rightTask;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = 
/*#__PURE__*/
flow(TE.fromIOEither, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReaderEither = (ma) => flow(ma, TE.fromEither);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = 
/*#__PURE__*/
ET.match(RT.Functor);
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchW = match;
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchE = 
/*#__PURE__*/
ET.matchE(RT.Monad);
/**
 * Less strict version of [`matchE`](#matchE).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchEW = matchE;
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/
ET.getOrElse(RT.Functor);
/**
 * Less strict version of [`getOrElse`](#getOrElse).
 *
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseW = getOrElse;
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = 
/*#__PURE__*/
ET.getOrElseE(RT.Monad);
/**
 * Less strict version of [`getOrElseE`](#getOrElseE).
 *
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseEW = getOrElseE;
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const toUnion = 
/*#__PURE__*/
ET.toUnion(RT.Functor);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/
ET.fromNullable(RT.Pointed);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = 
/*#__PURE__*/
ET.fromNullableK(RT.Pointed);
/**
 * @category interop
 * @since 3.0.0
 */
export const chainNullableK = 
/*#__PURE__*/
ET.chainNullableK(RT.Monad);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = R.local;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElse = 
/*#__PURE__*/
ET.orElse(RT.Monad);
/**
 * Less strict version of [`orElse`](#orElse).
 *
 * @category combinators
 * @since 3.0.0
 */
export const orElseW = orElse;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElseFirst = 
/*#__PURE__*/
ET.orElseFirst(RT.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElseFirstW = orElseFirst;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orLeft = 
/*#__PURE__*/
ET.orLeft(RT.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = 
/*#__PURE__*/
ET.swap(RT.Functor);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOEitherK = (f) => (...a) => fromIOEither(f(...a));
/**
 * Less strict version of [`chainIOEitherK`](#chainIOEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainIOEitherKW = (f) => chainW(fromIOEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainIOEitherK = chainIOEitherKW;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskEitherK = (f) => (...a) => fromTaskEither(f(...a));
/**
 * Less strict version of [`chainTaskEitherK`](#chainTaskEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainTaskEitherKW = (f) => chainW(fromTaskEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainTaskEitherK = chainTaskEitherKW;
/**
 * Less strict version of [`chainFirstTaskEitherK`](#chainFirstTaskEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstTaskEitherKW = (f) => chainFirstW(fromTaskEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstTaskEitherK = chainFirstTaskEitherKW;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderEitherK = (f) => flow(f, fromReaderEither);
/**
 * Less strict version of [`chainReaderEitherK`](#chainReaderEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderEitherKW = (f) => chainW(fromReaderEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderEitherK = chainReaderEitherKW;
/**
 * Less strict version of [`chainFirstReaderEitherK`](#chainFirstReaderEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderEitherKW = (f) => chainFirstW(fromReaderEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderEitherK = chainFirstReaderEitherKW;
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
ET.map(RT.Functor);
/**
 * Map a pair of functions over the two last type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = 
/*#__PURE__*/
ET.bimap(RT.Functor);
/**
 * Map a function over the second type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
ET.mapLeft(RT.Functor);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/
ET.ap(RT.ApplyPar);
/**
 * Less strict version of [`ap`](#ap).
 *
 * @category Apply
 * @since 3.0.0
 */
export const apW = ap;
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export const chain = 
/*#__PURE__*/
ET.chain(RT.Monad);
/**
 * Less strict version of [`chain`](#chain).
 *
 * @category Chain
 * @since 3.0.0
 */
export const chainW = chain;
/**
 * Less strict version of [`flatten`](#flatten).
 *
 * @category combinators
 * @since 3.0.0
 */
export const flattenW = 
/*#__PURE__*/
chainW(identity);
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const flatten = flattenW;
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export const alt = 
/*#__PURE__*/
ET.alt(RT.Monad);
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export const altW = alt;
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicativeReaderTaskValidation = (A, S) => ({
    map,
    ap: ap_(R.Apply, TE.getApplicativeTaskValidation(A, S)),
    of
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getAltReaderTaskValidation = (S) => {
    return {
        map,
        alt: ET.altValidation(RT.Monad, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getCompactable = (M) => {
    const C = E.getCompactable(M);
    return {
        compact: compact_(RT.Functor, C),
        separate: separate_(RT.Functor, C, E.Functor)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFilterable = (M) => {
    const F = E.getFilterable(M);
    return {
        filter: filter(RT.Functor, F),
        filterMap: filterMap(RT.Functor, F),
        partition: partition(RT.Functor, F),
        partitionMap: partitionMap(RT.Functor, F)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplyPar = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apFirst = 
/*#__PURE__*/
apFirst_(ApplyPar);
/**
 * Less strict version of [`apFirst`](#apfirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export const apFirstW = apFirst;
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apSecond = 
/*#__PURE__*/
apSecond_(ApplyPar);
/**
 * Less strict version of [`apSecond`](#apsecond).
 *
 * @category combinators
 * @since 3.0.0
 */
export const apSecondW = apSecond;
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativePar = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    map,
    chain
};
const apSeq = 
/*#__PURE__*/
apSeq_(Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplySeq = {
    map,
    ap: apSeq
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativeSeq = {
    map,
    ap: apSeq,
    of
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const chainFirst = 
/*#__PURE__*/
chainFirst_(Chain);
/**
 * Less strict version of [`chainFirst`](#chainFirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstW = chainFirst;
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/
fromIOK_(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainIOK = 
/*#__PURE__*/
chainIOK_(FromIO, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstIOK = 
/*#__PURE__*/
chainFirstIOK_(FromIO, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = 
/*#__PURE__*/
fromTaskK_(FromTask);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainTaskK = 
/*#__PURE__*/
chainTaskK_(FromTask, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstTaskK = 
/*#__PURE__*/
chainFirstTaskK_(FromTask, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader
};
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = 
/*#__PURE__*/
ask_(FromReader);
/**
 * Projects a value from the global context in a `ReaderEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = 
/*#__PURE__*/
asks_(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderK = 
/*#__PURE__*/
fromReaderK_(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderK = 
/*#__PURE__*/
chainReaderK_(FromReader, Chain);
/**
 * Less strict version of [`chainReaderK`](#chainReaderK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderKW = chainReaderK;
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderK = 
/*#__PURE__*/
chainFirstReaderK_(FromReader, Chain);
/**
 * Less strict version of [`chainFirstReaderK`](#chainFirstReaderK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderKW = chainFirstReaderK;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderTaskK = (f) => (...a) => rightReaderTask(f(...a));
/**
 * Less strict version of [`chainReaderTaskK`](#chainReaderTaskK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderTaskKW = (f) => chainW(fromReaderTaskK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderTaskK = chainReaderTaskKW;
/**
 * Less strict version of [`chainFirstReaderTaskK`](#chainFirstReaderTaskK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderTaskKW = (f) => chainFirstW(fromReaderTaskK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderTaskK = chainFirstReaderTaskKW;
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/
fromOption_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/
fromOptionK_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainOptionK = 
/*#__PURE__*/
chainOptionK_(FromEither, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherK = 
/*#__PURE__*/
chainEitherK_(FromEither, Chain);
/**
 * Less strict version of [`chainEitherK`](#chainEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherKW = chainEitherK;
/**
 * Less strict version of [`chainFirstEitherK`](#chainfirsteitherk).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherKW = (f) => chainFirstW(fromEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherK = chainFirstEitherKW;
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/
fromPredicate_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElse = 
/*#__PURE__*/
filterOrElse_(FromEither, Chain);
/**
 * Less strict version of [`filterOrElse`](#filterOrElse).
 *
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElseW = filterOrElse;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = 
/*#__PURE__*/
fromEitherK_(FromEither);
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Alt = {
    map,
    alt
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Make sure that a resource is cleaned up in the event of an exception (\*). The release action is called regardless of
 * whether the body action throws (\*) or returns.
 *
 * (\*) i.e. returns a `Left`
 *
 * @since 3.0.0
 */
export const bracket = 
/*#__PURE__*/
ET.bracket(RT.Monad);
/**
 * Less strict version of [`bracket`](#bracket).
 *
 * @since 3.0.0
 */
export const bracketW = bracket;
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = 
/*#__PURE__*/
of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/
bindTo_(Functor);
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/
bind_(Chain);
/**
 * Less strict version of [`bind`](#bind).
 *
 * @since 3.0.0
 */
export const bindW = bind;
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apS = 
/*#__PURE__*/
apS_(ApplyPar);
/**
 * Less strict version of [`apS`](#apS).
 *
 * @since 3.0.0
 */
export const apSW = apS;
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/
tupled_(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/
apT_(ApplyPar);
/**
 * Less strict version of [`apT`](#apT).
 *
 * @since 3.0.0
 */
export const apTW = apT;
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(R.traverseReadonlyNonEmptyArrayWithIndex(f), R.map(TE.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (f) => flow(R.traverseReadonlyNonEmptyArrayWithIndex(f), R.map(TE.traverseReadonlyNonEmptyArrayWithIndexSeq(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
