import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Returns a `Refinement` from a `Option` returning function.
 * This function ensures that a `Refinement` definition is type-safe.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromOptionK = (getOption) => {
    return (a) => _.isSome(getOption(a));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromEitherK = (getEither) => {
    return (a) => _.isRight(getEither(a));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const id = () => {
    return (_) => true;
};
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const not = (refinement) => (a) => !refinement(a);
/**
 * @category combinators
 * @since 3.0.0
 */
export const or = (second) => (first) => (a) => first(a) || second(a);
/**
 * @category combinators
 * @since 3.0.0
 */
export const and = (second) => (first) => (a) => first(a) && second(a);
/**
 * @category combinators
 * @since 3.0.0
 */
export const zero = () => {
    return (_) => false;
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const compose = (bc) => (ab) => {
    return (i) => ab(i) && bc(i);
};
