/**
 * @since 3.0.0
 */
import { map as map_, mapLeftDefault } from './Bifunctor';
import { flap as flap_ } from './Functor';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const separated = (left, right) => ({ left, right });
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = (f, g) => (fa) => separated(f(left(fa)), g(right(fa)));
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
mapLeftDefault(bimap);
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
map_(Bifunctor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const left = (s) => s.left;
/**
 * @since 3.0.0
 */
export const right = (s) => s.right;
