import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, apT as apT_ } from './Apply';
import { mapLeftDefault } from './Bifunctor';
import { bind as bind_, chainFirst as chainFirst_ } from './Chain';
import * as E from './Either';
import { chainEitherK as chainEitherK_, chainOptionK as chainOptionK_, filterOrElse as filterOrElse_, fromEitherK as fromEitherK_, fromOption as fromOption_, fromOptionK as fromOptionK_, fromPredicate as fromPredicate_, chainFirstEitherK as chainFirstEitherK_ } from './FromEither';
import { chainFirstIOK as chainFirstIOK_, chainIOK as chainIOK_, fromIOK as fromIOK_ } from './FromIO';
import { ask as ask_, asks as asks_, chainFirstReaderK as chainFirstReaderK_, chainReaderK as chainReaderK_, fromReaderK as fromReaderK_ } from './FromReader';
import { chainStateK as chainStateK_, fromStateK as fromStateK_, get as get_, gets as gets_, modify as modify_, put as put_ } from './FromState';
import { chainFirstTaskK as chainFirstTaskK_, chainTaskK as chainTaskK_, fromTaskK as fromTaskK_ } from './FromTask';
import { flow, identity, pipe } from './function';
import { bindTo as bindTo_, flap as flap_, tupled as tupled_ } from './Functor';
import * as _ from './internal';
import * as R from './Reader';
import * as RTE from './ReaderTaskEither';
import * as ST from './StateT';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = (e) => () => RTE.left(e);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = 
/*#__PURE__*/
ST.of(RTE.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = (ma) => fromReaderTaskEither(RTE.rightTask(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = (me) => fromReaderTaskEither(RTE.leftTask(me));
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReader = (ma) => fromReaderTaskEither(RTE.rightReader(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReader = (me) => fromReaderTaskEither(RTE.leftReader(me));
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = (ma) => fromReaderTaskEither(RTE.rightIO(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = (me) => fromReaderTaskEither(RTE.leftIO(me));
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightState = 
/*#__PURE__*/
ST.fromState(RTE.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftState = (me) => (s) => RTE.left(me(s)[0]);
/**
 * Less strict version of [`asksStateReaderTaskEitherK`](#asksstatereadertaskeitherk).
 *
 * @category constructors
 * @since 3.0.0
 */
export const asksStateReaderTaskEitherW = (f) => (s) => (r) => f(r)(s)(r);
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksStateReaderTaskEither = asksStateReaderTaskEitherW;
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = 
/*#__PURE__*/
E.match((e) => left(e), right);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReader = rightReader;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTask = rightTask;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromState = 
/*#__PURE__*/
ST.fromState(RTE.Pointed);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTaskEither = (ma) => fromReaderTaskEither(RTE.fromTaskEither(ma));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = (ma) => fromReaderTaskEither(RTE.fromIOEither(ma));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReaderEither = (ma) => fromReaderTaskEither(RTE.fromReaderEither(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReaderTaskEither = 
/*#__PURE__*/
ST.fromF(RTE.Functor);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = (f) => (ma) => flow(ma, R.local(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOEitherK = (f) => (...a) => fromIOEither(f(...a));
/**
 * Less strict version of [`chainIOEitherK`](#chainIOEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainIOEitherKW = (f) => (ma) => pipe(ma, chainW(fromIOEitherK(f)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainIOEitherK = chainIOEitherKW;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskEitherK = (f) => (...a) => fromTaskEither(f(...a));
/**
 * Less strict version of [`chainTaskEitherK`](#chainTaskEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainTaskEitherKW = (f) => (ma) => pipe(ma, chainW(fromTaskEitherK(f)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainTaskEitherK = chainTaskEitherKW;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderTaskEitherK = (f) => (...a) => fromReaderTaskEither(f(...a));
/**
 * Less strict version of [`chainReaderTaskEitherK`](#chainReaderTaskEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderTaskEitherKW = (f) => (ma) => pipe(ma, chainW(fromReaderTaskEitherK(f)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderTaskEitherK = chainReaderTaskEitherKW;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
ST.map(RTE.Functor);
/**
 * Map a pair of functions over the two last type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = (f, g) => (fea) => (s) => pipe(fea(s), RTE.bimap(f, ([a, s]) => [g(a), s]));
/**
 * Map a function over the third type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
mapLeftDefault(bimap);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/
ST.ap(RTE.Monad);
/**
 * Less strict version of [`ap`](#ap).
 *
 * @category Apply
 * @since 3.0.0
 */
export const apW = ap;
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export const chain = 
/*#__PURE__*/
ST.chain(RTE.Monad);
/**
 * Less strict version of [`chain`](#chain).
 *
 * @category Chain
 * @since 3.0.0
 */
export const chainW = chain;
/**
 * Less strict version of [`flatten`](#flatten).
 *
 * @category combinators
 * @since 3.0.0
 */
export const flattenW = 
/*#__PURE__*/
chainW(identity);
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const flatten = flattenW;
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export const altW = (second) => (first) => (r) => pipe(first(r), RTE.altW(() => second()(r)));
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export const alt = altW;
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apFirst = 
/*#__PURE__*/
apFirst_(Apply);
/**
 * Less strict version of [`apFirst`](#apfirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export const apFirstW = apFirst;
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apSecond = 
/*#__PURE__*/
apSecond_(Apply);
/**
 * Less strict version of [`apSecond`](#apsecond).
 *
 * @category combinators
 * @since 3.0.0
 */
export const apSecondW = apSecond;
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Alt = {
    map,
    alt
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    map,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromState = {
    fromState
};
/**
 * Get the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export const get = 
/*#__PURE__*/
get_(FromState);
/**
 * Set the state
 *
 * @category constructors
 * @since 3.0.0
 */
export const put = 
/*#__PURE__*/
put_(FromState);
/**
 * Modify the state by applying a function to the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export const modify = 
/*#__PURE__*/
modify_(FromState);
/**
 * Get a value which depends on the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export const gets = 
/*#__PURE__*/
gets_(FromState);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromStateK = 
/*#__PURE__*/
fromStateK_(FromState);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainStateK = 
/*#__PURE__*/
chainStateK_(FromState, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    chain
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const chainFirst = 
/*#__PURE__*/
chainFirst_(Chain);
/**
 * Less strict version of [`chainFirst`](#chainFirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstW = chainFirst;
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/
fromIOK_(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainIOK = 
/*#__PURE__*/
chainIOK_(FromIO, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstIOK = 
/*#__PURE__*/
chainFirstIOK_(FromIO, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = 
/*#__PURE__*/
fromTaskK_(FromTask);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainTaskK = 
/*#__PURE__*/
chainTaskK_(FromTask, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstTaskK = 
/*#__PURE__*/
chainFirstTaskK_(FromTask, Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader
};
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = 
/*#__PURE__*/
ask_(FromReader);
/**
 * Projects a value from the global context in a `ReaderEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = 
/*#__PURE__*/
asks_(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderK = 
/*#__PURE__*/
fromReaderK_(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderK = 
/*#__PURE__*/
chainReaderK_(FromReader, Chain);
/**
 * Less strict version of [`chainReaderK`](#chainReaderK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainReaderKW = chainReaderK;
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderK = 
/*#__PURE__*/
chainFirstReaderK_(FromReader, Chain);
/**
 * Less strict version of [`chainFirstReaderK`](#chainFirstReaderK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstReaderKW = chainFirstReaderK;
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/
fromOption_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/
fromOptionK_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainOptionK = 
/*#__PURE__*/
chainOptionK_(FromEither, Chain);
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherK = 
/*#__PURE__*/
chainEitherK_(FromEither, Chain);
/**
 * Less strict version of [`chainEitherK`](#chainEitherK).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherKW = chainEitherK;
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherK = 
/*#__PURE__*/
chainFirstEitherK_(FromEither, Chain);
/**
 * Less strict version of [`chainFirstEitherK`](#chainfirsteitherk).
 *
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherKW = chainFirstEitherK;
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/
fromPredicate_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElse = 
/*#__PURE__*/
filterOrElse_(FromEither, Chain);
/**
 * Less strict version of [`filterOrElse`](#filterOrElse).
 *
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElseW = filterOrElse;
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = 
/*#__PURE__*/
fromEitherK_(FromEither);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Run a computation in the `StateReaderTaskEither` monad, discarding the final state
 *
 * @since 3.0.0
 */
export const evaluate = 
/*#__PURE__*/
ST.evaluate(RTE.Functor);
/**
 * Run a computation in the `StateReaderTaskEither` monad discarding the result
 *
 * @since 3.0.0
 */
export const execute = 
/*#__PURE__*/
ST.execute(RTE.Functor);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/
bindTo_(Functor);
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/
bind_(Chain);
/**
 * Less strict version of [`bind`](#bind).
 *
 * @since 3.0.0
 */
export const bindW = bind;
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apS = 
/*#__PURE__*/
apS_(Apply);
/**
 * Less strict version of [`apS`](#apS).
 *
 * @since 3.0.0
 */
export const apSW = apS;
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/
tupled_(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/
apT_(Apply);
/**
 * Less strict version of [`apT`](#apT).
 *
 * @since 3.0.0
 */
export const apTW = apT;
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => (as) => (s) => (r) => () => _.tail(as).reduce((acc, a, i) => acc.then((ebs) => _.isLeft(ebs)
    ? acc
    : f(i + 1, a)(ebs.right[1])(r)().then((eb) => {
        if (_.isLeft(eb)) {
            return eb;
        }
        const [b, s] = eb.right;
        ebs.right[0].push(b);
        ebs.right[1] = s;
        return ebs;
    })), f(0, _.head(as))(s)(r)().then(E.map(([b, s]) => [[b], s])));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : of(_.emptyReadonlyArray));
};
