import { identity, pipe } from './function';
import { flap as flap_ } from './Functor';
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => ({
    peek: (s) => f(fa.peek(s)),
    pos: fa.pos
});
/**
 * @category Extend
 * @since 3.0.0
 */
export const extend = (f) => (wa) => ({
    peek: (s) => f({ peek: wa.peek, pos: s }),
    pos: wa.pos
});
/**
 * @category Extract
 * @since 3.0.0
 */
export const extract = (wa) => wa.peek(wa.pos);
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const duplicate = 
/*#__PURE__*/
extend(identity);
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Reposition the focus at the specified position.
 *
 * @since 3.0.0
 */
export const seek = (s) => (wa) => ({ peek: wa.peek, pos: s });
/**
 * Reposition the focus at the specified position, which depends on the current position.
 *
 * @since 3.0.0
 */
export const seeks = (f) => (wa) => ({ peek: wa.peek, pos: f(wa.pos) });
/**
 * Extract a value from a position which depends on the current position.
 *
 * @since 3.0.0
 */
export const peeks = (f) => (wa) => wa.peek(f(wa.pos));
export function experiment(F) {
    return (f) => (wa) => pipe(f(wa.pos), F.map((s) => wa.peek(s)));
}
