import { fromEitherK as fromEitherK_, fromOption as fromOption_, fromOptionK as fromOptionK_, fromPredicate as fromPredicate_ } from './FromEither';
import { fromIOK as fromIOK_ } from './FromIO';
import { fromTaskK as fromTaskK_ } from './FromTask';
import { fromTheseK as fromTheseK_ } from './FromThese';
import { flow, SK } from './function';
import { flap as flap_ } from './Functor';
import * as _ from './internal';
import * as T from './Task';
import * as TH from './These';
import * as TT from './TheseT';
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = 
/*#__PURE__*/
TT.left(T.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = 
/*#__PURE__*/
TT.right(T.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const both = 
/*#__PURE__*/
TT.both(T.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = 
/*#__PURE__*/
TT.rightF(T.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = 
/*#__PURE__*/
TT.leftF(T.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = 
/*#__PURE__*/
flow(T.fromIO, rightTask);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = 
/*#__PURE__*/
flow(T.fromIO, leftTask);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = 
/*#__PURE__*/
T.fromIO;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromEither = T.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromThese = T.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTask = rightTask;
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = 
/*#__PURE__*/
TT.match(T.Functor);
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchW = match;
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchE = 
/*#__PURE__*/
TT.matchE(T.Monad);
/**
 * Less strict version of [`matchE`](#matchE).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchEW = matchE;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = 
/*#__PURE__*/
TT.swap(T.Functor);
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
TT.map(T.Functor);
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = 
/*#__PURE__*/
TT.bimap(T.Functor);
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
TT.mapLeft(T.Functor);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (A, S) => ({
    map,
    ap: TT.ap(A, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (A, S) => {
    const AS = getApply(A, S);
    return {
        map,
        ap: AS.ap,
        of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getChain = (S) => ({
    map,
    chain: TT.chain(T.Monad, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (S) => {
    const C = getChain(S);
    return {
        map,
        of,
        chain: C.chain
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/
fromOption_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/
fromOptionK_(FromEither);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/
fromPredicate_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = 
/*#__PURE__*/
fromEitherK_(FromEither);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromThese = {
    fromThese
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTheseK = 
/*#__PURE__*/
fromTheseK_(FromThese);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/
fromIOK_(FromIO);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = 
/*#__PURE__*/
fromTaskK_(FromTask);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const toTuple2 = 
/*#__PURE__*/
TT.toTuple2(T.Functor);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (S) => {
    const g = TH.traverseReadonlyNonEmptyArrayWithIndex(S);
    return (f) => flow(T.traverseReadonlyNonEmptyArrayWithIndex(f), T.map(g(SK)));
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (S) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(S)(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (S) => (f) => (as) => () => _.tail(as).reduce((acc, a, i) => acc.then((ebs) => TH.isLeft(ebs)
    ? acc
    : f(i + 1, a)().then((eb) => {
        if (TH.isLeft(eb)) {
            return eb;
        }
        if (TH.isBoth(eb)) {
            const right = ebs.right;
            right.push(eb.right);
            return TH.isBoth(ebs) ? TH.both(S.concat(eb.left)(ebs.left), right) : TH.both(eb.left, right);
        }
        ebs.right.push(eb.right);
        return ebs;
    })), f(0, _.head(as))().then(TH.map(_.singleton)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (S) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(S)(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
