import { flap as flap_ } from './Functor';
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => (p) => f(fa(p));
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const getComonad = (monoid) => ({
    map,
    extend: (f) => (wa) => (p1) => f((p2) => wa(monoid.concat(p2)(p1))),
    extract: (wa) => wa(monoid.empty)
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Extracts a value at a relative position which depends on the current value.
 *
 * @since 3.0.0
 */
export const tracks = (M) => (f) => (wa) => wa(f(wa(M.empty)));
/**
 * Get the current position.
 *
 * @since 3.0.0
 */
export const listen = (wa) => (p) => [wa(p), p];
/**
 * Get a value which depends on the current position.
 *
 * @since 3.0.0
 */
export const listens = (f) => (wa) => (p) => [
    wa(p),
    f(p)
];
/**
 * Apply a function to the current position.
 *
 * @since 3.0.0
 */
export const censor = (f) => (wa) => (p) => wa(f(p));
