/**
 * @since 3.0.0
 */
import type { IO } from './IO'
import * as E from './Eq'
import * as O from './Ord'
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Eq: E.Eq<Date>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const EqDate: E.Eq<Date>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const EqMonth: E.Eq<Date>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const EqYear: E.Eq<Date>
/**
 * @example
 * import { Ord } from 'fp-ts/Date'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(new Date(1, 1, 2020), Ord.compare(new Date(1, 1, 2021))), -1)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const Ord: O.Ord<Date>
/**
 * Returns the current `Date`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const create: IO<Date>
/**
 * Returns the number of milliseconds elapsed since January 1, 1970, 00:00:00 UTC.
 *
 * @since 3.0.0
 */
export declare const now: IO<number>
