/**
 * @since 3.0.0
 */
import type { Alt1 } from './Alt'
import type { Applicative1 } from './Applicative'
import { Apply1 } from './Apply'
import { Chain1 } from './Chain'
import { ChainRec1 } from './ChainRec'
import type { Comonad1 } from './Comonad'
import type { Eq } from './Eq'
import type { Extend1 } from './Extend'
import type { Foldable1 } from './Foldable'
import { Functor1 } from './Functor'
import type { Monad1 } from './Monad'
import type { Pointed1 } from './Pointed'
import type { Show } from './Show'
import type { Traversable1 } from './Traversable'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Identity<A> = A
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor1<URI>['map']
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: Apply1<URI>['ap']
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: Pointed1<URI>['of']
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: Chain1<URI>['chain']
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: Extend1<URI>['extend']
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: Comonad1<URI>['extract']
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <A>(ma: Identity<A>) => Identity<Identity<A>>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: Identity<Identity<A>>) => Identity<A>
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduce: Foldable1<URI>['reduce']
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const foldMap: Foldable1<URI>['foldMap']
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: Foldable1<URI>['reduceRight']
/**
 * @since 3.0.0
 */
export declare const traverse: Traversable1<URI>['traverse']
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export declare const altW: <B>(second: () => Identity<B>) => <A>(first: Identity<A>) => Identity<A | B>
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export declare const alt: Alt1<URI>['alt']
/**
 * @category ChainRec
 * @since 3.0.0
 */
export declare const chainRec: ChainRec1<URI>['chainRec']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Identity'
declare module './HKT' {
  interface URItoKind<A> {
    readonly Identity: Identity<A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: Eq<A>) => Eq<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor1<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: (a: A) => B) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply1<URI>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: B) => <A>(first: A) => A
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: B) => <A>(first: A) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad1<URI>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => B) => (first: A) => A
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ChainRec: ChainRec1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: Foldable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: Traversable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alt: Alt1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad1<URI>
/**
 * @since 3.0.0
 */
export declare const Do: Identity<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(name: N) => <A>(fa: A) => { [K in N]: A }
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (ma: A) => { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: B
) => (fa: A) => { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }
/**
 * @since 3.0.0
 */
export declare const ApT: Identity<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: A) => readonly [A]
/**
 * @since 3.0.0
 */
export declare const apT: <B>(fb: B) => <A extends readonly unknown[]>(fas: A) => readonly [...A, B]
