/**
 * @since 3.0.0
 */
import { Either } from './Either'
/**
 * @since 3.0.0
 */
export declare type Json = boolean | number | string | null | JsonArray | JsonRecord
/**
 * @since 3.0.0
 */
export interface JsonRecord {
  readonly [key: string]: Json
}
/**
 * @since 3.0.0
 */
export interface JsonArray extends ReadonlyArray<Json> {}
/**
 * Converts a JavaScript Object Notation (JSON) string into an object.
 *
 * @example
 * import * as J from 'fp-ts/Json'
 * import * as E from 'fp-ts/Either'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe('{"a":1}', J.parse), E.right({ a: 1 }))
 * assert.deepStrictEqual(pipe('{"a":}', J.parse), E.left(new SyntaxError('Unexpected token } in JSON at position 5')))
 *
 * @since 3.0.0
 */
export declare const parse: (s: string) => Either<unknown, Json>
/**
 * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.
 *
 * @since 3.0.0
 */
export declare const stringify: <A>(a: A) => Either<unknown, string>
