/**
 * @since 3.0.0
 */
import * as E from './Eq'
import * as M from './Monoid'
import * as S from './Semigroup'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Ordering = -1 | 0 | 1
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const reverse: (o: Ordering) => Ordering
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const matchW: <A, B, C>(
  onLessThan: () => A,
  onEqual: () => B,
  onGreaterThan: () => C
) => (o: Ordering) => A | B | C
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <A>(onLessThan: () => A, onEqual: () => A, onGreaterThan: () => A) => (o: Ordering) => A
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Eq: E.Eq<Ordering>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Semigroup: S.Semigroup<Ordering>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monoid: M.Monoid<Ordering>
/**
 * @since 3.0.0
 */
export declare const sign: (n: number) => Ordering
