import type { Compactable2 } from './Compactable'
import type { Either } from './Either'
import type { Endomorphism } from './Endomorphism'
import { Eq } from './Eq'
import type { Filterable2 } from './Filterable'
import type { FilterableWithIndex2C } from './FilterableWithIndex'
import type { Foldable, Foldable1, Foldable2, Foldable2C, Foldable3, Foldable4 } from './Foldable'
import type { FoldableWithIndex2C } from './FoldableWithIndex'
import { Functor2 } from './Functor'
import type { FunctorWithIndex2C } from './FunctorWithIndex'
import type { HKT, Kind, Kind2, Kind3, Kind4, URIS, URIS2, URIS3, URIS4 } from './HKT'
import type { Magma } from './Magma'
import type { Monoid } from './Monoid'
import * as O from './Option'
import type { Ord } from './Ord'
import type { Semigroup } from './Semigroup'
import { Separated } from './Separated'
import type { Show } from './Show'
import type { Traversable2C } from './Traversable'
import type { TraversableWithIndex2C } from './TraversableWithIndex'
import type { Unfoldable, Unfoldable1 } from './Unfoldable'
import { Witherable2C } from './Witherable'
import Option = O.Option
/**
 * Create a `ReadonlyMap` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const singleton: <K, A>(k: K, a: A) => ReadonlyMap<K, A>
/**
 * Create a `ReadonlyRecord` from a `Foldable` collection of key/value pairs, using the
 * specified `Magma` to combine values for duplicate keys, and the specified `f` to map to key/value pairs.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function fromFoldable<F extends URIS4>(
  F: Foldable4<F>
): <K, B>(
  E: Eq<K>,
  M: Magma<B>
) => <A>(f: (a: A) => readonly [K, B]) => <S, R, E>(fka: Kind4<F, S, R, E, A>) => ReadonlyMap<K, B>
export declare function fromFoldable<F extends URIS3>(
  F: Foldable3<F>
): <K, B>(
  E: Eq<K>,
  M: Magma<B>
) => <A>(f: (a: A) => readonly [K, B]) => <R, E>(fka: Kind3<F, R, E, A>) => ReadonlyMap<K, B>
export declare function fromFoldable<F extends URIS2>(
  F: Foldable2<F>
): <K, B>(E: Eq<K>, M: Magma<B>) => <A>(f: (a: A) => readonly [K, B]) => <E>(fka: Kind2<F, E, A>) => ReadonlyMap<K, B>
export declare function fromFoldable<F extends URIS>(
  F: Foldable1<F>
): <K, B>(E: Eq<K>, M: Magma<B>) => <A>(f: (a: A) => readonly [K, B]) => (fka: Kind<F, A>) => ReadonlyMap<K, B>
export declare function fromFoldable<F>(
  F: Foldable<F>
): <K, B>(E: Eq<K>, M: Magma<B>) => <A>(f: (a: A) => readonly [K, B]) => (fka: HKT<F, A>) => ReadonlyMap<K, B>
/**
 * Insert an element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const insertAt: <K>(E: Eq<K>) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => O.Option<ReadonlyMap<K, A>>
/**
 * Insert or replace a key/value pair in a `ReadonlyMap`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const upsertAt: <K>(E: Eq<K>) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * Change the element at the specified keys, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export declare const updateAt: <K>(E: Eq<K>) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => O.Option<ReadonlyMap<K, A>>
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export declare const modifyAt: <K>(
  E: Eq<K>
) => <A>(k: K, f: Endomorphism<A>) => (m: ReadonlyMap<K, A>) => O.Option<ReadonlyMap<K, A>>
/**
 * Delete the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const deleteAt: <K>(E: Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => O.Option<ReadonlyMap<K, A>>
/**
 * Delete a key and value from a `ReadonlyMap`, returning the value as well as the subsequent `ReadonlyMap`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pop: <K>(
  E: Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => O.Option<readonly [A, ReadonlyMap<K, A>]>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export declare const mapWithIndex: <K, A, B>(f: (k: K, a: A) => B) => (m: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: Compactable2<URI>['compact']
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: Compactable2<URI>['separate']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filter: Filterable2<URI>['filter']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: Filterable2<URI>['filterMap']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partition: Filterable2<URI>['partition']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: Filterable2<URI>['partitionMap']
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function filterWithIndex<K, A, B extends A>(
  predicateWithIndex: (k: K, a: A) => a is B
): (m: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
export declare function filterWithIndex<K, A>(
  predicateWithIndex: (k: K, a: A) => boolean
): <B extends A>(m: ReadonlyMap<K, B>) => ReadonlyMap<K, B>
export declare function filterWithIndex<K, A>(
  predicateWithIndex: (k: K, a: A) => boolean
): (m: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare const filterMapWithIndex: <K, A, B>(
  f: (k: K, a: A) => O.Option<B>
) => (fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function partitionWithIndex<K, A, B extends A>(
  predicateWithIndex: (k: K, a: A) => a is B
): (m: ReadonlyMap<K, A>) => Separated<ReadonlyMap<K, A>, ReadonlyMap<K, B>>
export declare function partitionWithIndex<K, A>(
  predicateWithIndex: (k: K, a: A) => boolean
): <B extends A>(m: ReadonlyMap<K, B>) => Separated<ReadonlyMap<K, B>, ReadonlyMap<K, B>>
export declare function partitionWithIndex<K, A>(
  predicateWithIndex: (k: K, a: A) => boolean
): (m: ReadonlyMap<K, A>) => Separated<ReadonlyMap<K, A>, ReadonlyMap<K, A>>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare const partitionMapWithIndex: <K, A, B, C>(
  f: (k: K, a: A) => Either<B, C>
) => (fa: ReadonlyMap<K, A>) => Separated<ReadonlyMap<K, B>, ReadonlyMap<K, C>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'ReadonlyMap'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly ReadonlyMap: ReadonlyMap<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <K, A>(SK: Show<K>, SA: Show<A>) => Show<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <K, A>(EK: Eq<K>, EA: Eq<A>) => Eq<ReadonlyMap<K, A>>
/**
 * Get a `Monoid` instance for `ReadonlyMap` given a `Semigroup` instance for its values.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <K, A>(EK: Eq<K>, SA: Semigroup<A>) => Monoid<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: ReadonlyMap<E, (a: A) => B>) => ReadonlyMap<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFunctorWithIndex: <K = never>() => FunctorWithIndex2C<'ReadonlyMap', K, K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: Compactable2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: Filterable2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFilterableWithIndex: <K = never>() => FilterableWithIndex2C<'ReadonlyMap', K, K>
/**
 * @since 3.0.0
 */
export declare const reduce: <K>(O: Ord<K>) => <B, A>(b: B, f: (b: B, a: A) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: <K>(O: Ord<K>) => <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: ReadonlyMap<K, A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRight: <K>(O: Ord<K>) => <B, A>(b: B, f: (a: A, b: B) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldable: <K>(O: Ord<K>) => Foldable2C<'ReadonlyMap', K>
/**
 * @since 3.0.0
 */
export declare const reduceWithIndex: <K>(
  O: Ord<K>
) => <B, A>(b: B, f: (i: K, b: B, a: A) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMapWithIndex: <K>(
  O: Ord<K>
) => <M>(M: Monoid<M>) => <A>(f: (i: K, a: A) => M) => (fa: ReadonlyMap<K, A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRightWithIndex: <K>(
  O: Ord<K>
) => <B, A>(b: B, f: (i: K, a: A, b: B) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldableWithIndex: <K>(O: Ord<K>) => FoldableWithIndex2C<'ReadonlyMap', K, K>
/**
 * @since 3.0.0
 */
export declare const traverse: <K>(O: Ord<K>) => import('./Traversable').Traverse2C<'ReadonlyMap', K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversable: <K>(O: Ord<K>) => Traversable2C<'ReadonlyMap', K>
/**
 * @since 3.0.0
 */
export declare const traverseWithIndex: <K>(
  O: Ord<K>
) => import('./TraversableWithIndex').TraverseWithIndex2C<'ReadonlyMap', K, K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversableWithIndex: <K>(O: Ord<K>) => TraversableWithIndex2C<'ReadonlyMap', K, K>
/**
 * @since 3.0.0
 */
export declare const wither: <K>(O: Ord<K>) => import('./Witherable').Wither2C<'ReadonlyMap', K>
/**
 * @since 3.0.0
 */
export declare const wilt: <K>(O: Ord<K>) => import('./Witherable').Wilt2C<'ReadonlyMap', K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getWitherable: <K>(O: Ord<K>) => Witherable2C<'ReadonlyMap', K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionSemigroup: <K, A>(E: Eq<K>, S: Semigroup<A>) => Semigroup<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionMonoid: <K, A>(E: Eq<K>, S: Semigroup<A>) => Monoid<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getIntersectionSemigroup: <K, A>(E: Eq<K>, S: Semigroup<A>) => Semigroup<ReadonlyMap<K, A>>
/**
 * @category combinator
 * @since 3.0.0
 */
export declare const getDifferenceMagma: <K>(E: Eq<K>) => <A>() => Magma<ReadonlyMap<K, A>>
/**
 * Calculate the number of key/value pairs in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const size: <K, A>(d: ReadonlyMap<K, A>) => number
/**
 * Test whether or not a `ReadonlyMap` is empty.
 *
 * @since 3.0.0
 */
export declare const isEmpty: <K, A>(d: ReadonlyMap<K, A>) => boolean
/**
 * Test whether or not a key exists in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const member: <K>(E: Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => boolean
/**
 * Tests whether a value is a member of a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: Eq<A>) => (a: A) => <K>(m: ReadonlyMap<K, A>) => boolean
/**
 * Get a sorted `ReadonlyArray` of the keys contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const keys: <K>(O: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => readonly K[]
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const values: <A>(O: Ord<A>) => <K>(m: ReadonlyMap<K, A>) => readonly A[]
/**
 * @since 3.0.0
 */
export declare const collect: <K>(O: Ord<K>) => <A, B>(f: (k: K, a: A) => B) => (m: ReadonlyMap<K, A>) => readonly B[]
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 * If the result is a `Some`, the existing key is also returned.
 *
 * @since 3.0.0
 */
export declare const lookupWithKey: <K>(E: Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => O.Option<readonly [K, A]>
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const lookup: <K>(E: Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => O.Option<A>
/**
 * Test whether or not one `ReadonlyMap` contains all of the keys and values contained in another `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const isSubmap: <K, A>(
  EK: Eq<K>,
  SA: Eq<A>
) => (second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => boolean
/**
 * An empty `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const empty: ReadonlyMap<never, never>
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const toReadonlyArray: <K>(O: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => readonly (readonly [K, A])[]
/**
 * Unfolds a `ReadonlyMap` into a data structure of key/value pairs.
 *
 * @since 3.0.0
 */
export declare function toUnfoldable<F extends URIS>(
  U: Unfoldable1<F>
): <K>(o: Ord<K>) => <A>(d: ReadonlyMap<K, A>) => Kind<F, readonly [K, A]>
export declare function toUnfoldable<F>(
  U: Unfoldable<F>
): <K>(o: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => HKT<F, readonly [K, A]>
/**
 * @since 3.0.0
 */
export declare const union: <K, A>(
  E: Eq<K>,
  M: Magma<A>
) => (second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @since 3.0.0
 */
export declare const intersection: <K, A>(
  E: Eq<K>,
  M: Magma<A>
) => (second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @since 3.0.0
 */
export declare const difference: <K>(
  E: Eq<K>
) => <A>(_second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
