/**
 * @since 3.0.0
 */
import type { Applicative, Applicative1, Applicative2, Applicative2C, Applicative3, Applicative3C } from './Applicative'
import type { Compactable1 } from './Compactable'
import type { Either } from './Either'
import type { Endomorphism } from './Endomorphism'
import { Eq } from './Eq'
import type { Filterable1 } from './Filterable'
import type { FilterableWithIndex1, PredicateWithIndex, RefinementWithIndex } from './FilterableWithIndex'
import type { Foldable as Foldable_, Foldable1, Foldable2, Foldable3, Foldable4 } from './Foldable'
import type { FoldableWithIndex1 } from './FoldableWithIndex'
import { Functor1 } from './Functor'
import type { FunctorWithIndex1 } from './FunctorWithIndex'
import type { HKT, Kind, Kind2, Kind3, Kind4, URIS, URIS2, URIS3, URIS4 } from './HKT'
import type { Magma } from './Magma'
import type { Monoid } from './Monoid'
import * as O from './Option'
import { Ord } from './Ord'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import type { Semigroup } from './Semigroup'
import { Separated } from './Separated'
import type { Show } from './Show'
import type { Traversable1 } from './Traversable'
import type { TraversableWithIndex1 } from './TraversableWithIndex'
import type { Unfoldable, Unfoldable1 } from './Unfoldable'
import { Witherable1 } from './Witherable'
import Option = O.Option
/**
 * @category model
 * @since 3.0.0
 */
export declare type ReadonlyRecord<K extends string, T> = Readonly<Record<K, T>>
/**
 * Create a `ReadonlyRecord` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const singleton: <A>(k: string, a: A) => Readonly<Record<string, A>>
/**
 * Create a `ReadonlyRecord` from a `Foldable` collection of key/value pairs, using the
 * specified `Magma` to combine values for duplicate keys, and the specified `f` to map to key/value pairs.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function fromFoldable<F extends URIS4>(
  F: Foldable4<F>
): <B>(
  M: Magma<B>
) => <A>(f: (a: A) => readonly [string, B]) => <S, R, E>(r: Kind4<F, S, R, E, A>) => ReadonlyRecord<string, B>
export declare function fromFoldable<F extends URIS3>(
  F: Foldable3<F>
): <B>(
  M: Magma<B>
) => <A>(f: (a: A) => readonly [string, B]) => <R, E>(r: Kind3<F, R, E, A>) => ReadonlyRecord<string, B>
export declare function fromFoldable<F extends URIS2>(
  F: Foldable2<F>
): <B>(M: Magma<B>) => <A>(f: (a: A) => readonly [string, B]) => <E>(r: Kind2<F, E, A>) => ReadonlyRecord<string, B>
export declare function fromFoldable<F extends URIS>(
  F: Foldable1<F>
): <B>(M: Magma<B>) => <A>(f: (a: A) => readonly [string, B]) => (r: Kind<F, A>) => ReadonlyRecord<string, B>
export declare function fromFoldable<F>(
  F: Foldable_<F>
): <B>(M: Magma<B>) => <A>(f: (a: A) => readonly [string, B]) => (r: HKT<F, A>) => ReadonlyRecord<string, B>
/**
 * Insert an element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const insertAt: <A>(
  k: string,
  a: A
) => (r: Readonly<Record<string, A>>) => O.Option<Readonly<Record<string, A>>>
/**
 * Insert or replace a key/value pair in a `ReadonlyRecord`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const upsertAt: <A>(k: string, a: A) => (r: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * Change the element at the specified keys, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const updateAt: <A>(
  k: string,
  a: A
) => (r: Readonly<Record<string, A>>) => O.Option<Readonly<Record<string, A>>>
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const modifyAt: <A>(
  k: string,
  f: Endomorphism<A>
) => (r: Readonly<Record<string, A>>) => O.Option<Readonly<Record<string, A>>>
/**
 * Delete the element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const deleteAt: (
  k: string
) => <A>(r: Readonly<Record<string, A>>) => O.Option<Readonly<Record<string, A>>>
/**
 * Delete the element at the specified key, returning the value as well as the subsequent `ReadonlyRecord`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pop: (
  k: string
) => <A>(r: Readonly<Record<string, A>>) => O.Option<readonly [A, Readonly<Record<string, A>>]>
/**
 * Map a `ReadonlyRecord` passing the values to the iterating function.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(
  f: (a: A) => B
) => <K extends string>(r: Readonly<Record<K, A>>) => Readonly<Record<K, B>>
/**
 * Map a `ReadonlyRecord` passing both the keys and values to the iterating function.
 *
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export declare function mapWithIndex<K extends string, A, B>(
  f: (k: K, a: A) => B
): (r: ReadonlyRecord<K, A>) => ReadonlyRecord<K, B>
/**
 * @since 3.0.0
 */
export declare const keys: (O: Ord<string>) => <K extends string>(r: Readonly<Record<K, unknown>>) => readonly K[]
/**
 * @since 3.0.0
 */
export declare const reduceWithIndex: (
  O: Ord<string>
) => <B, K extends string, A>(b: B, f: (k: K, b: B, a: A) => B) => (r: Readonly<Record<K, A>>) => B
/**
 * @since 3.0.0
 */
export declare const foldMapWithIndex: (
  O: Ord<string>
) => <M>(M: Monoid<M>) => <K extends string, A>(f: (k: K, a: A) => M) => (r: Readonly<Record<K, A>>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRightWithIndex: (
  O: Ord<string>
) => <B, K extends string, A>(b: B, f: (k: K, a: A, b: B) => B) => (r: Readonly<Record<K, A>>) => B
/**
 * @since 3.0.0
 */
export declare function traverseWithIndex(
  O: Ord<string>
): {
  <F extends URIS3>(F: Applicative3<F>): <K extends string, A, R, E, B>(
    f: (k: K, a: A) => Kind3<F, R, E, B>
  ) => (ta: ReadonlyRecord<K, A>) => Kind3<F, R, E, ReadonlyRecord<K, B>>
  <F extends URIS3, E>(F: Applicative3C<F, E>): <K extends string, A, R, B>(
    f: (k: K, a: A) => Kind3<F, R, E, B>
  ) => (ta: ReadonlyRecord<K, A>) => Kind3<F, R, E, ReadonlyRecord<K, B>>
  <F extends URIS2>(F: Applicative2<F>): <K extends string, A, E, B>(
    f: (k: K, a: A) => Kind2<F, E, B>
  ) => (ta: ReadonlyRecord<K, A>) => Kind2<F, E, ReadonlyRecord<K, B>>
  <F extends URIS2, E>(F: Applicative2C<F, E>): <K extends string, A, B>(
    f: (k: K, a: A) => Kind2<F, E, B>
  ) => (ta: ReadonlyRecord<K, A>) => Kind2<F, E, ReadonlyRecord<K, B>>
  <F extends URIS>(F: Applicative1<F>): <K extends string, A, B>(
    f: (k: K, a: A) => Kind<F, B>
  ) => (ta: ReadonlyRecord<K, A>) => Kind<F, ReadonlyRecord<K, B>>
  <F>(F: Applicative<F>): <K extends string, A, B>(
    f: (k: K, a: A) => HKT<F, B>
  ) => (ta: ReadonlyRecord<K, A>) => HKT<F, ReadonlyRecord<K, B>>
}
/**
 * @since 3.0.0
 */
export declare function traverse(
  O: Ord<string>
): {
  <F extends URIS3>(F: Applicative3<F>): <A, R, E, B>(
    f: (a: A) => Kind3<F, R, E, B>
  ) => <K extends string>(ta: ReadonlyRecord<K, A>) => Kind3<F, R, E, ReadonlyRecord<K, B>>
  <F extends URIS3, E>(F: Applicative3C<F, E>): <A, R, B>(
    f: (a: A) => Kind3<F, R, E, B>
  ) => <K extends string>(ta: ReadonlyRecord<K, A>) => Kind3<F, R, E, ReadonlyRecord<K, B>>
  <F extends URIS2>(F: Applicative2<F>): <A, E, B>(
    f: (a: A) => Kind2<F, E, B>
  ) => <K extends string>(ta: ReadonlyRecord<K, A>) => Kind2<F, E, ReadonlyRecord<K, B>>
  <F extends URIS2, E>(F: Applicative2C<F, E>): <A, B>(
    f: (a: A) => Kind2<F, E, B>
  ) => <K extends string>(ta: ReadonlyRecord<K, A>) => Kind2<F, E, ReadonlyRecord<K, B>>
  <F extends URIS>(F: Applicative1<F>): <A, B>(
    f: (a: A) => Kind<F, B>
  ) => <K extends string>(ta: ReadonlyRecord<K, A>) => Kind<F, ReadonlyRecord<K, B>>
  <F>(F: Applicative<F>): <A, B>(
    f: (a: A) => HKT<F, B>
  ) => <K extends string>(ta: ReadonlyRecord<K, A>) => HKT<F, ReadonlyRecord<K, B>>
}
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function partitionMapWithIndex<K extends string, A, B, C>(
  f: (key: K, a: A) => Either<B, C>
): (r: ReadonlyRecord<K, A>) => Separated<ReadonlyRecord<string, B>, ReadonlyRecord<string, C>>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function partitionWithIndex<K extends string, A, B extends A>(
  refinementWithIndex: RefinementWithIndex<K, A, B>
): (r: ReadonlyRecord<K, A>) => Separated<ReadonlyRecord<string, A>, ReadonlyRecord<string, B>>
export declare function partitionWithIndex<K extends string, A>(
  predicateWithIndex: PredicateWithIndex<K, A>
): <B extends A>(r: ReadonlyRecord<K, B>) => Separated<ReadonlyRecord<string, B>, ReadonlyRecord<string, B>>
export declare function partitionWithIndex<K extends string, A>(
  predicateWithIndex: PredicateWithIndex<K, A>
): (r: ReadonlyRecord<K, A>) => Separated<ReadonlyRecord<string, A>, ReadonlyRecord<string, A>>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function filterMapWithIndex<K extends string, A, B>(
  f: (key: K, a: A) => Option<B>
): (r: ReadonlyRecord<K, A>) => ReadonlyRecord<string, B>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function filterWithIndex<K extends string, A, B extends A>(
  refinementWithIndex: RefinementWithIndex<K, A, B>
): (r: ReadonlyRecord<K, A>) => ReadonlyRecord<string, B>
export declare function filterWithIndex<K extends string, A>(
  predicateWithIndex: PredicateWithIndex<K, A>
): <B extends A>(r: ReadonlyRecord<K, B>) => ReadonlyRecord<string, B>
export declare function filterWithIndex<K extends string, A>(
  predicateWithIndex: PredicateWithIndex<K, A>
): (r: ReadonlyRecord<K, A>) => ReadonlyRecord<string, A>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filter: Filterable1<URI>['filter']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: Filterable1<URI>['filterMap']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partition: Filterable1<URI>['partition']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: Filterable1<URI>['partitionMap']
/**
 * @since 3.0.0
 */
export declare const reduce: (
  O: Ord<string>
) => <B, A>(b: B, f: (b: B, a: A) => B) => (r: Readonly<Record<string, A>>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: (
  O: Ord<string>
) => <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (r: Readonly<Record<string, A>>) => M
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: (
  O: Ord<string>
) => <B, A>(b: B, f: (a: A, b: B) => B) => (r: Readonly<Record<string, A>>) => B
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: Compactable1<URI>['compact']
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: Compactable1<URI>['separate']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'ReadonlyRecord'
declare module './HKT' {
  interface URItoKind<A> {
    readonly ReadonlyRecord: ReadonlyRecord<string, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: (O: Ord<string>) => <A>(S: Show<A>) => Show<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare function getEq<A, K extends string>(E: Eq<A>): Eq<ReadonlyRecord<K, A>>
/**
 * Returns a `Monoid` instance for `ReadonlyRecord`s given a `Semigroup` instance for their values.
 *
 * @example
 * import * as N from 'fp-ts/number'
 * import { getMonoid } from 'fp-ts/ReadonlyRecord'
 * import { pipe } from 'fp-ts/function'
 *
 * const M = getMonoid(N.SemigroupSum)
 * assert.deepStrictEqual(pipe({ foo: 123 }, M.concat({ foo: 456 })), { foo: 579 })
 *
 * @category instances
 * @since 3.0.0
 */
export declare function getMonoid<A, K extends string>(S: Semigroup<A>): Monoid<ReadonlyRecord<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor1<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Readonly<Record<string, (a: A) => B>>) => Readonly<Record<string, B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FunctorWithIndex: FunctorWithIndex1<URI, string>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldable: (O: Ord<string>) => Foldable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldableWithIndex: (O: Ord<string>) => FoldableWithIndex1<URI, string>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: Compactable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: Filterable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FilterableWithIndex: FilterableWithIndex1<URI, string>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversable: (O: Ord<string>) => Traversable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversableWithIndex: (O: Ord<string>) => TraversableWithIndex1<URI, string>
/**
 * @category Witherable
 * @since 3.0.0
 */
export declare const wither: (O: Ord<string>) => Witherable1<URI>['wither']
/**
 * @category Witherable
 * @since 3.0.0
 */
export declare const wilt: (O: Ord<string>) => Witherable1<URI>['wilt']
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getWitherable: (O: Ord<string>) => Witherable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionSemigroup: <A>(S: Semigroup<A>) => Semigroup<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionMonoid: <A>(S: Semigroup<A>) => Monoid<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getIntersectionSemigroup: <A>(S: Semigroup<A>) => Semigroup<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getDifferenceMagma: <A>() => Magma<Readonly<Record<string, A>>>
/**
 * Calculate the number of key/value pairs in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const size: <A>(r: Readonly<Record<string, A>>) => number
/**
 * Test whether a `ReadonlyRecord` is empty.
 *
 * @since 3.0.0
 */
export declare const isEmpty: <A>(r: Readonly<Record<string, A>>) => boolean
/**
 * Map a `ReadonlyRecord` into an `ReadonlyArray`.
 *
 * @example
 * import { collect } from 'fp-ts/ReadonlyRecord'
 * import * as S from 'fp-ts/string'
 *
 * const x: { a: string, b: boolean } = { a: 'foo', b: false }
 * assert.deepStrictEqual(
 *   collect(S.Ord)((key, val) => ({key: key, value: val}))(x),
 *   [{key: 'a', value: 'foo'}, {key: 'b', value: false}]
 * )
 *
 * @since 3.0.0
 */
export declare const collect: (
  O: Ord<string>
) => <K extends string, A, B>(f: (k: K, a: A) => B) => (r: Readonly<Record<K, A>>) => readonly B[]
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const toReadonlyArray: (
  O: Ord<string>
) => <K extends string, A>(r: Readonly<Record<K, A>>) => readonly (readonly [K, A])[]
/**
 * Unfolds a `ReadonlyRecord` into a data structure of key/value pairs.
 *
 * @since 3.0.0
 */
export declare function toUnfoldable(
  O: Ord<string>
): {
  <F extends URIS>(U: Unfoldable1<F>): <K extends string, A>(r: ReadonlyRecord<K, A>) => Kind<F, readonly [K, A]>
  <F>(U: Unfoldable<F>): <K extends string, A>(r: ReadonlyRecord<K, A>) => HKT<F, readonly [K, A]>
}
/**
 * Test whether or not a key exists in a `ReadonlyRecord`.
 *
 * Note. This function is not pipeable because is a custom type guard.
 *
 * @since 3.0.0
 */
export declare const has: <K extends string>(k: string, r: Readonly<Record<K, unknown>>) => k is K
/**
 * Test whether one `ReadonlyRecord` contains all of the keys and values contained in another `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const isSubrecord: <A>(
  E: Eq<A>
) => (second: Readonly<Record<string, A>>) => (first: Readonly<Record<string, A>>) => boolean
/**
 * Lookup the value for a key in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const lookup: (k: string) => <A>(r: Readonly<Record<string, A>>) => O.Option<A>
/**
 * An empty `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const empty: ReadonlyRecord<string, never>
/**
 * @since 3.0.0
 */
export declare function every<A, B extends A>(
  refinement: Refinement<A, B>
): Refinement<ReadonlyRecord<string, A>, ReadonlyRecord<string, B>>
export declare function every<A>(predicate: Predicate<A>): Predicate<ReadonlyRecord<string, A>>
/**
 * @since 3.0.0
 */
export declare const some: <A>(predicate: (a: A) => boolean) => (r: Readonly<Record<string, A>>) => boolean
/**
 * Tests whether a value is a member of a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: Eq<A>) => (a: A) => (r: Readonly<Record<string, A>>) => boolean
/**
 * @since 3.0.0
 */
export declare const union: <A>(
  M: Magma<A>
) => (second: Readonly<Record<string, A>>) => (first: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * @since 3.0.0
 */
export declare const intersection: <A>(
  M: Magma<A>
) => (second: Readonly<Record<string, A>>) => (first: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * @since 3.0.0
 */
export declare const difference: <A>(
  second: Readonly<Record<string, A>>
) => (first: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * Converts a `ReadonlyRecord` into a `ReadonlyArray` of `[key, value]` tuples.
 *
 * @example
 * import { toEntries } from 'fp-ts/ReadonlyRecord'
 *
 * assert.deepStrictEqual(toEntries({ a: 1, b: 2 }), [['a', 1], ['b', 2]])
 *
 * @since 3.0.0
 */
export declare const toEntries: <K extends string, A>(r: Readonly<Record<K, A>>) => readonly (readonly [K, A])[]
/**
 * Converts a `ReadonlyArray` of `[key, value]` tuples into a `ReadonlyRecord`.
 *
 * @example
 * import { fromEntries } from 'fp-ts/ReadonlyRecord'
 *
 * assert.deepStrictEqual(fromEntries([['a', 1], ['b', 2], ['a', 3]]), { b: 2, a: 3 })
 *
 * @since 3.0.0
 */
export declare const fromEntries: <A>(fa: readonly (readonly [string, A])[]) => Record<string, A>
