/**
 * @since 3.0.0
 */
import { Bifunctor2 } from './Bifunctor'
import { Functor2 } from './Functor'
/**
 * @since 3.0.0
 */
export interface Separated<E, A> {
  readonly left: E
  readonly right: A
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const separated: <E, A>(left: E, right: A) => Separated<E, A>
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const bimap: Bifunctor2<URI>['bimap']
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapLeft: Bifunctor2<URI>['mapLeft']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Separated'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Separated: Separated<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor2<URI>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Separated<E, (a: A) => B>) => Separated<E, B>
/**
 * @since 3.0.0
 */
export declare const left: <E, A>(s: Separated<E, A>) => E
/**
 * @since 3.0.0
 */
export declare const right: <E, A>(s: Separated<E, A>) => A
