/**
 * @since 3.0.0
 */
import type { Comonad2 } from './Comonad'
import type { Endomorphism } from './Endomorphism'
import type { Extend2 } from './Extend'
import { Functor as Functor_, Functor1, Functor2, Functor2C, Functor3, Functor3C, Functor4 } from './Functor'
import type { HKT, Kind, Kind2, Kind3, Kind4, URIS, URIS2, URIS3, URIS4 } from './HKT'
/**
 * @category model
 * @since 3.0.0
 */
export interface Store<S, A> {
  readonly peek: (s: S) => A
  readonly pos: S
}
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: Extend2<URI>['extend']
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: Comonad2<URI>['extract']
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <E, A>(wa: Store<E, A>) => Store<E, Store<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Store'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Store: Store<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Store<E, (a: A) => B>) => Store<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad2<URI>
/**
 * Reposition the focus at the specified position.
 *
 * @since 3.0.0
 */
export declare const seek: <S>(s: S) => <A>(wa: Store<S, A>) => Store<S, A>
/**
 * Reposition the focus at the specified position, which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const seeks: <S>(f: Endomorphism<S>) => <A>(wa: Store<S, A>) => Store<S, A>
/**
 * Extract a value from a position which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const peeks: <S>(f: Endomorphism<S>) => <A>(wa: Store<S, A>) => A
/**
 * Extract a collection of values from positions which depend on the current position.
 *
 * @since 3.0.0
 */
export declare function experiment<F extends URIS4>(
  F: Functor4<F>
): <S1, S2, R, E>(f: (s: S1) => Kind4<F, S2, R, E, S1>) => <A>(wa: Store<S1, A>) => Kind4<F, S2, R, E, A>
export declare function experiment<F extends URIS3>(
  F: Functor3<F>
): <S, R, E>(f: (s: S) => Kind3<F, R, E, S>) => <A>(wa: Store<S, A>) => Kind3<F, R, E, A>
export declare function experiment<F extends URIS3, E>(
  F: Functor3C<F, E>
): <S, R>(f: (s: S) => Kind3<F, R, E, S>) => <A>(wa: Store<S, A>) => Kind3<F, R, E, A>
export declare function experiment<F extends URIS2>(
  F: Functor2<F>
): <S, E>(f: (s: S) => Kind2<F, E, S>) => <A>(wa: Store<S, A>) => Kind2<F, E, A>
export declare function experiment<F extends URIS2, E>(
  F: Functor2C<F, E>
): <S>(f: (s: S) => Kind2<F, E, S>) => <A>(wa: Store<S, A>) => Kind2<F, E, A>
export declare function experiment<F extends URIS>(
  F: Functor1<F>
): <S>(f: (s: S) => Kind<F, S>) => <A>(wa: Store<S, A>) => Kind<F, A>
export declare function experiment<F>(F: Functor_<F>): <S>(f: (s: S) => HKT<F, S>) => <A>(wa: Store<S, A>) => HKT<F, A>
