/**
 * @since 3.0.0
 */
import type { Applicative2C } from './Applicative'
import type { Apply1, Apply2C } from './Apply'
import type { Bifunctor2 } from './Bifunctor'
import type { Chain2C } from './Chain'
import { FromEither2 } from './FromEither'
import { FromIO2 } from './FromIO'
import { FromTask2 } from './FromTask'
import { FromThese2 } from './FromThese'
import { Functor2 } from './Functor'
import type { IO } from './IO'
import type { URI as IEURI } from './IOEither'
import type { Monad2C } from './Monad'
import { NaturalTransformation22 } from './NaturalTransformation'
import type { Pointed2 } from './Pointed'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Semigroup } from './Semigroup'
import * as T from './Task'
import * as TH from './These'
import These = TH.These
import Task = T.Task
/**
 * @category model
 * @since 3.0.0
 */
export interface TaskThese<E, A> extends Task<These<E, A>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const left: <E, A = never>(e: E) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const right: <A, E = never>(a: A) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const both: <E, A>(e: E, a: A) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightTask: <A, E = never>(ma: Task<A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftTask: <E, A = never>(me: Task<E>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightIO: <A, E = never>(ma: IO<A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftIO: <E, A = never>(me: IO<E>) => TaskThese<E, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: NaturalTransformation22<IEURI, URI>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromEither: FromEither2<URI>['fromEither']
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromThese: FromThese2<URI>['fromThese']
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: FromIO2<URI>['fromIO']
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: FromTask2<URI>['fromTask']
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <E, B, A>(
  onLeft: (e: E) => B,
  onRight: (a: A) => B,
  onBoth: (e: E, a: A) => B
) => (ma: T.Task<TH.These<E, A>>) => T.Task<B>
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const matchW: <E, B, A, C, D>(
  onLeft: (e: E) => B,
  onRight: (a: A) => C,
  onBoth: (e: E, a: A) => D
) => (ma: T.Task<TH.These<E, A>>) => T.Task<B | C | D>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchE: <E, B, A>(
  onLeft: (e: E) => T.Task<B>,
  onRight: (a: A) => T.Task<B>,
  onBoth: (e: E, a: A) => T.Task<B>
) => (ma: T.Task<TH.These<E, A>>) => T.Task<B>
/**
 * Less strict version of [`matchE`](#matchE).
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const matchEW: <E, B, A, C, D>(
  onLeft: (e: E) => T.Task<B>,
  onRight: (a: A) => T.Task<C>,
  onBoth: (e: E, a: A) => T.Task<D>
) => (ma: T.Task<TH.These<E, A>>) => T.Task<B | C | D>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <E, A>(ma: T.Task<TH.These<E, A>>) => T.Task<TH.These<A, E>>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const bimap: Bifunctor2<URI>['bimap']
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapLeft: Bifunctor2<URI>['mapLeft']
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A, E = never>(a: A) => TaskThese<E, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'TaskThese'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly TaskThese: TaskThese<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <E>(A: Apply1<T.URI>, S: Semigroup<E>) => Apply2C<'TaskThese', E>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <E>(A: Apply1<T.URI>, S: Semigroup<E>) => Applicative2C<'TaskThese', E>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getChain: <E>(S: Semigroup<E>) => Chain2C<'TaskThese', E>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <E>(S: Semigroup<E>) => Monad2C<'TaskThese', E>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: TaskThese<E, (a: A) => B>) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: FromEither2<URI>
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <E>(
  onNone: import('./function').Lazy<E>
) => import('./NaturalTransformation').NaturalTransformation12C<'Option', 'TaskThese', E>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <E>(
  onNone: import('./function').Lazy<E>
) => <A extends readonly unknown[], B>(f: (...a: A) => import('./Option').Option<B>) => (...a: A) => TaskThese<E, B>
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: {
  <A, B extends A>(refinement: import('./Refinement').Refinement<A, B>): (a: A) => TaskThese<A, B>
  <A_1>(predicate: import('./Predicate').Predicate<A_1>): <B_1 extends A_1>(b: B_1) => TaskThese<B_1, B_1>
  <A_2>(predicate: import('./Predicate').Predicate<A_2>): (a: A_2) => TaskThese<A_2, A_2>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends readonly unknown[], E, B>(
  f: (...a: A) => import('./Either').Either<E, B>
) => (...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromThese: FromThese2<URI>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTheseK: <A extends readonly unknown[], E, B>(
  f: (...a: A) => TH.These<E, B>
) => (...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: FromIO2<URI>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends readonly unknown[], B>(
  f: (...a: A) => IO<B>
) => <E>(...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromTask: FromTask2<URI>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskK: <A extends readonly unknown[], B>(
  f: (...a: A) => T.Task<B>
) => <E>(...a: A) => TaskThese<E, B>
/**
 * @since 3.0.0
 */
export declare const toTuple2: <E, A>(
  e: import('./function').Lazy<E>,
  a: import('./function').Lazy<A>
) => (fa: T.Task<TH.These<E, A>>) => T.Task<readonly [E, A]>
/**
 * @since 3.0.0
 */
export declare const ApT: TaskThese<never, readonly []>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <E>(
  S: Semigroup<E>
) => <A, B>(
  f: (index: number, a: A) => TaskThese<E, B>
) => (as: ReadonlyNonEmptyArray<A>) => TaskThese<E, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <E>(
  S: Semigroup<E>
) => <A, B>(f: (index: number, a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndexSeq: <E>(
  S: Semigroup<E>
) => <A, B>(
  f: (index: number, a: A) => TaskThese<E, B>
) => (as: ReadonlyNonEmptyArray<A>) => TaskThese<E, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndexSeq: <E>(
  S: Semigroup<E>
) => <A, B>(f: (index: number, a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
