/**
 * @since 3.0.0
 */
import type { Comonad2C } from './Comonad'
import { Functor2 } from './Functor'
import type { Monoid } from './Monoid'
/**
 * @category model
 * @since 3.0.0
 */
export interface Traced<P, A> {
  (p: P): A
}
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Traced'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Traced: Traced<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Traced<E, (a: A) => B>) => Traced<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getComonad: <P>(monoid: Monoid<P>) => Comonad2C<'Traced', P>
/**
 * Extracts a value at a relative position which depends on the current value.
 *
 * @since 3.0.0
 */
export declare const tracks: <P>(M: Monoid<P>) => <A>(f: (a: A) => P) => (wa: Traced<P, A>) => A
/**
 * Get the current position.
 *
 * @since 3.0.0
 */
export declare const listen: <P, A>(wa: Traced<P, A>) => Traced<P, readonly [A, P]>
/**
 * Get a value which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const listens: <P, B>(f: (p: P) => B) => <A>(wa: Traced<P, A>) => Traced<P, readonly [A, B]>
/**
 * Apply a function to the current position.
 *
 * @since 3.0.0
 */
export declare const censor: <P>(f: (p: P) => P) => <A>(wa: Traced<P, A>) => Traced<P, A>
