/**
 * @since 3.0.0
 */
import type { Applicative2C } from './Applicative'
import type { Apply2C } from './Apply'
import { Bifunctor2 } from './Bifunctor'
import type { Chain2C } from './Chain'
import type { ChainRec2C } from './ChainRec'
import type { Comonad2 } from './Comonad'
import type { Extend2 } from './Extend'
import type { Foldable2 } from './Foldable'
import { Functor2 } from './Functor'
import type { Monad2C } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed2C } from './Pointed'
import type { Semigroup } from './Semigroup'
import type { Semigroupoid2 } from './Semigroupoid'
import type { Traversable2 } from './Traversable'
/**
 * @since 3.0.0
 */
export declare type Tuple2<E, A> = readonly [A, E]
/**
 * @since 3.0.0
 */
export declare const tuple2: <A, E>(a: A, e: E) => Tuple2<E, A>
/**
 * @since 3.0.0
 */
export declare const fst: <A, E>(t: Tuple2<E, A>) => A
/**
 * @since 3.0.0
 */
export declare const snd: <A, E>(t: Tuple2<E, A>) => E
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <A, E>(t: Tuple2<E, A>) => Tuple2<A, E>
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const bimap: <E, G, A, B>(mapSnd: (e: E) => G, mapFst: (a: A) => B) => (t: Tuple2<E, A>) => Tuple2<G, B>
/**
 * Map a function over the second component of a `Tuple2`.
 *
 * This is the `mapLeft` operation of the `Bifunctor` instance.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapSnd: Bifunctor2<URI>['mapLeft']
/**
 * @category Semigroupoid
 * @since 3.0.0
 */
export declare const compose: Semigroupoid2<URI>['compose']
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: Extend2<URI>['extend']
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: Comonad2<URI>['extract']
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <E, A>(t: Tuple2<E, A>) => Tuple2<E, Tuple2<E, A>>
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduce: Foldable2<URI>['reduce']
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const foldMap: Foldable2<URI>['foldMap']
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: Foldable2<URI>['reduceRight']
/**
 * @since 3.0.0
 */
export declare const traverse: Traversable2<URI>['traverse']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Tuple2'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Tuple2: Tuple2<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor2<URI>
/**
 * Map a function over the first component of a `Tuple2`.
 *
 * This is the `map` operation of the `Functor` instance.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const mapFst: Functor2<URI>['map']
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Tuple2<E, (a: A) => B>) => Tuple2<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Semigroupoid: Semigroupoid2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: Foldable2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: Traversable2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getPointed: <M>(M: Monoid<M>) => Pointed2C<'Tuple2', M>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <S>(S: Semigroup<S>) => Apply2C<'Tuple2', S>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <M>(M: Monoid<M>) => Applicative2C<'Tuple2', M>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getChain: <S>(S: Semigroup<S>) => Chain2C<'Tuple2', S>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <M>(M: Monoid<M>) => Monad2C<'Tuple2', M>
/**
 * @category instances
 * @since 3.0.0
 */
export declare function getChainRec<M>(M: Monoid<M>): ChainRec2C<URI, M>
