/**
 * @since 3.0.0
 */
import type { Applicative2C } from './Applicative'
import type { Apply2C } from './Apply'
import type { Chain2C } from './Chain'
import { Functor2 } from './Functor'
import type { Monad2C } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed2C } from './Pointed'
import type { Semigroup } from './Semigroup'
/**
 * @category model
 * @since 3.0.0
 */
export interface Writer<W, A> {
  (): readonly [A, W]
}
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <W>(w: W) => Writer<W, void>
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const listen: <W, A>(fa: Writer<W, A>) => Writer<W, readonly [A, W]>
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pass: <W, A>(fa: Writer<W, readonly [A, (w: W) => W]>) => Writer<W, A>
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const listens: <W, B>(f: (w: W) => B) => <A>(fa: Writer<W, A>) => Writer<W, readonly [A, B]>
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const censor: <W>(f: (w: W) => W) => <A>(fa: Writer<W, A>) => Writer<W, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Writer'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Writer: Writer<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getPointed: <W>(M: Monoid<W>) => Pointed2C<'Writer', W>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <W>(S: Semigroup<W>) => Apply2C<'Writer', W>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <W>(M: Monoid<W>) => Applicative2C<'Writer', W>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getChain: <W>(S: Semigroup<W>) => Chain2C<'Writer', W>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <W>(M: Monoid<W>) => Monad2C<'Writer', W>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Writer<E, (a: A) => B>) => Writer<E, B>
/**
 * @since 3.0.0
 */
export declare const evaluate: <W, A>(fa: Writer<W, A>) => A
/**
 * @since 3.0.0
 */
export declare const execute: <W, A>(fa: Writer<W, A>) => W
