/**
 * @since 3.0.0
 */
import * as B from './Bounded'
import * as E from './Eq'
import * as F from './Field'
import * as O from './Ord'
import * as S from './Show'
import type { Semigroup } from './Semigroup'
import type { Monoid } from './Monoid'
import type { Magma } from './Magma'
import { Refinement } from './Refinement'
/**
 * @category refinements
 * @since 3.0.0
 */
export declare const isNumber: Refinement<unknown, number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Eq: E.Eq<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Ord: O.Ord<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bounded: B.Bounded<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Field: F.Field<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Show: S.Show<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const MagmaSub: Magma<number>
/**
 * `number` semigroup under addition.
 *
 * @example
 * import { SemigroupSum } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupSum.concat(3)), 5)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupSum: Semigroup<number>
/**
 * `number` semigroup under multiplication.
 *
 * @example
 * import { SemigroupProduct } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupProduct.concat(3)), 6)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupProduct: Semigroup<number>
/**
 * `number` monoid under addition.
 *
 * The `empty` value is `0`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidSum: Monoid<number>
/**
 * `number` monoid under multiplication.
 *
 * The `empty` value is `1`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidProduct: Monoid<number>
